@@ -1578,6 +1578,7 @@ bool create_artifact(object_type *o_ptr, bool a_scroll)
 				warrior_artifact_bias = 40;
 				break;
 			case CLASS_WARRIOR_MAGE:
+			case CLASS_DEVICE_USER:
 				artifact_bias = BIAS_MAGE;
 				warrior_artifact_bias = 40;
 				break;
@@ -2524,6 +2525,7 @@ bool activate_random_artifact(object_type * o_ptr)
 #else
 			msg_print("It glows extremely brightly...");
 #endif
+			(void)hp_player(50);
 			(void)set_poisoned(0);
 			(void)set_confused(0);
 			(void)set_blind(0);
