@@ -541,6 +541,31 @@ sprintf(dummy, "%s{Ƥ}Ȥޤ", o_name);
 }
 
 
+void swap_wielding_hands(void)
+{
+	/* Swap hands wielding sub weapon */
+	if (is_two_handed())
+	{
+		char o_name[MAX_NLEN];
+		object_type *o_ptr = &inventory[INVEN_ARM];
+		object_type *q_ptr = &inventory[INVEN_WIELD];
+
+		object_copy(q_ptr, o_ptr);
+		object_wipe(o_ptr);
+
+		/* Describe local object */
+		object_desc(o_name, q_ptr, 0);
+
+#ifdef JP
+		msg_format("%s򱦼ǹ", o_name);
+#else
+		msg_format("You are weilding %s in your right hand.", o_name);
+#endif
+
+		p_ptr->redraw |= (PR_EQUIPPY);
+	}
+}
+
 
 /*
  * Take off an item
@@ -600,25 +625,7 @@ void do_cmd_takeoff(void)
 	(void)inven_takeoff(item, 255);
 
 	/* Swap hands wielding sub weapon */
-	if ((item == INVEN_WIELD) && is_two_handed())
-	{
-		char o_name[MAX_NLEN];
-		object_type *q_ptr = &inventory[INVEN_WIELD];
-
-		o_ptr = &inventory[INVEN_ARM];
-
-		object_copy(q_ptr, o_ptr);
-		object_wipe(o_ptr);
-
-		/* Describe local object */
-		object_desc(o_name, q_ptr, 0);
-
-#ifdef JP
-		msg_format("%s򱦼ǹ", o_name);
-#else
-		msg_format("You are weilding %s in your right hand.", o_name);
-#endif
-	}
+	if (item == INVEN_WIELD) swap_wielding_hands();
 
 	p_ptr->redraw |= (PR_EQUIPPY);
 }
@@ -700,7 +707,8 @@ static bool high_level_book(object_type *o_ptr)
 {
 	if ((o_ptr->tval == TV_LIFE_BOOK) ||
 	    (o_ptr->tval == TV_SORCERY_BOOK) ||
-	    (o_ptr->tval == TV_MUSOU_BOOK))
+	    (o_ptr->tval == TV_MUSOU_BOOK) ||
+		(o_ptr->tval == TV_MAGIC_BOOK))	/* Magic */
 	{
 		if (o_ptr->sval > 1)
 			return TRUE;
