@@ -799,6 +799,13 @@ void do_cmd_quaff_potion(void)
 
 	if (!get_item(&item, q, s, (USE_INVEN | USE_FLOOR))) return;
 
+	/* Magic */
+	if (p_ptr->keep_magic)
+	{
+		if (!(p_ptr->keep_spells & MS_INHAIL_POTION))
+			(void)stop_magic_spell_all();
+	}
+
 	/* Quaff the potion */
 	do_cmd_quaff_potion_aux(item);
 }
@@ -1344,6 +1351,18 @@ void do_cmd_read_scroll(void)
 
 	if (!get_item(&item, q, s, (USE_INVEN | USE_FLOOR))) return;
 
+	/* Magic */
+	if (p_ptr->keep_magic)
+	{
+		if (p_ptr->lev < 35)
+			(void)stop_magic_spell_all();
+		else
+		{
+			if (keeping_magic_spell_fully())
+				if (!stop_magic_spell()) return;
+		}
+	}
+
 	/* Read the scroll */
 	do_cmd_read_scroll_aux(item);
 }
@@ -1424,7 +1443,10 @@ static void do_cmd_use_staff_aux(int item)
 
 
 	/* Take a turn */
-	energy_use = 100;
+	if (p_ptr->pclass == CLASS_DEVICE_USER)
+		energy_use = 90 - p_ptr->lev;
+	else
+		energy_use = 100;
 
 	/* Not identified yet */
 	ident = FALSE;
@@ -1596,6 +1618,7 @@ static void do_cmd_use_staff_aux(int item)
 
 		case SV_STAFF_CURING:
 		{
+			if (hp_player(50)) ident = TRUE;
 			if (set_blind(0)) ident = TRUE;
 			if (set_poisoned(0)) ident = TRUE;
 			if (set_confused(0)) ident = TRUE;
@@ -1713,6 +1736,37 @@ static void do_cmd_use_staff_aux(int item)
 
 			break;
 		}
+
+		case SV_STAFF_WISHING:
+		{
+			int i, r;
+			int prob = (p_ptr->lev * 2);
+
+#ifdef JP
+			msg_print("ʤ˾ߤΤΤ롪");
+#else
+			msg_print("You may wish for an object.");
+#endif
+
+			ident = TRUE;
+
+			for (i = 0; i < 5; i++)
+			{
+				r = do_cmd_wishing(prob, TRUE, TRUE);
+				if (r >= 0) break;
+			}
+
+			if ((r < 1) && (!p_ptr->blind))	/* failed */
+			{
+#ifdef JP
+				msg_print("åդϰ֥֥롼˵...");
+#else
+				msg_print("The staff glows blue for a moment...");
+#endif
+			}
+
+			break;
+		}
 	}
 
 	/* Combine / Reorder the pack (later) */
@@ -1870,7 +1924,10 @@ static void do_cmd_aim_wand_aux(int item)
 
 
 	/* Take a turn */
-	energy_use = 100;
+	if (p_ptr->pclass == CLASS_DEVICE_USER)
+		energy_use = 90 - p_ptr->lev;
+	else
+		energy_use = 100;
 
 	/* Not identified yet */
 	ident = FALSE;
@@ -2171,6 +2228,20 @@ static void do_cmd_aim_wand_aux(int item)
 			ident = TRUE;
 			break;
 		}
+
+		case SV_WAND_STRIKING:
+		{
+			fire_bolt(GF_METEOR, dir, damroll(15 + p_ptr->lev / 3, 13));
+			ident = TRUE;
+			break;
+		}
+
+		case SV_WAND_ELEC_BOLT:
+		{
+			fire_bolt_or_beam(20, GF_ELEC, dir, damroll(5 + p_ptr->lev / 9, 8));
+			ident = TRUE;
+			break;
+		}
 	}
 
 
@@ -2287,7 +2358,10 @@ static void do_cmd_zap_rod_aux(int item)
 
 
 	/* Take a turn */
-	energy_use = 100;
+	if (p_ptr->pclass == CLASS_DEVICE_USER)
+		energy_use = 90 - p_ptr->lev;
+	else
+		energy_use = 100;
 
 	/* Not identified yet */
 	ident = FALSE;
@@ -2399,6 +2473,7 @@ static void do_cmd_zap_rod_aux(int item)
 
 		case SV_ROD_CURING:
 		{
+			if (hp_player(50)) ident = TRUE;
 			if (set_blind(0)) ident = TRUE;
 			if (set_poisoned(0)) ident = TRUE;
 			if (set_confused(0)) ident = TRUE;
@@ -2748,7 +2823,10 @@ static void do_cmd_activate_aux(int item)
 	}
 
 	/* Take a turn */
-	energy_use = 100;
+	if (p_ptr->pclass == CLASS_DEVICE_USER)
+		energy_use = 100;
+	else
+		energy_use = 100;
 
 	/* Extract the item level */
 	lev = get_object_level(o_ptr);
@@ -3741,3 +3819,70 @@ msg_print("𤷤Ƥɤʤ");
 		}
 	}
 }
+
+
+/*
+ * Use magic devide for device-users
+ */
+void do_cmd_use_device(void)
+{
+	int         item;
+	object_type *o_ptr;
+	cptr        q, s;
+
+	/* Prepare the hook */
+	item_tester_hook = item_tester_hook_recharge;
+
+	/* Get an item */
+#ifdef JP
+	q = "ɤȤޤ";
+	s = "ȤΤޤ";
+#else
+	q = "Use which item? ";
+	s = "You have nothing to use.";
+#endif
+
+	if (!get_item(&item, q, s, (USE_INVEN | USE_EQUIP | USE_FLOOR))) return;
+
+	/* Get the item (in the pack) */
+	if (item >= 0)
+	{
+		o_ptr = &inventory[item];
+	}
+	/* Get the item (on the floor) */
+	else
+	{
+		o_ptr = &o_list[0 - item];
+	}
+
+	switch (o_ptr->tval)
+	{
+		/* Aim a wand */
+		case TV_WAND:
+		{
+			do_cmd_aim_wand_aux(item);
+			break;
+		}
+
+		/* Use a staff */
+		case TV_STAFF:
+		{
+			do_cmd_use_staff_aux(item);
+			break;
+		}
+
+		/* Zap a rod */
+		case TV_ROD:
+		{
+			do_cmd_zap_rod_aux(item);
+			break;
+		}
+
+		/* Activate an artifact */
+		default:
+		{
+			do_cmd_activate_aux(item);
+			break;
+		}
+	}
+}
