@@ -686,34 +686,56 @@ static void regenhp(int percent)
  */
 static void regenmana(int percent)
 {
-	s32b        new_mana, new_mana_frac;
-	int                   old_csp;
+	s32b new_mana, new_mana_frac;
+	int old_csp = p_ptr->csp;
 
-	old_csp = p_ptr->csp;
-	new_mana = ((long)p_ptr->msp) * percent + PY_REGEN_MNBASE;
-	p_ptr->csp += (s16b)(new_mana >> 16);	/* div 65536 */
+	if (percent > 0)
+	{
+		s32b add_mana  = ((long)p_ptr->msp) * percent + PY_REGEN_MNBASE + p_ptr->csp_frac;
 
-	/* check for overflow */
-	if ((p_ptr->csp < 0) && (old_csp > 0))
-	{
-		p_ptr->csp = MAX_SHORT;
-	}
-	new_mana_frac = (new_mana & 0xFFFF) + p_ptr->csp_frac;	/* mod 65536 */
-	if (new_mana_frac >= 0x10000L)
-	{
-		p_ptr->csp_frac = (u16b)(new_mana_frac - 0x10000L);
-		p_ptr->csp++;
-	}
-	else
-	{
+		new_mana = (add_mana >> 16);	/* div 65536 */
+		new_mana_frac = (add_mana & 0xFFFF);	/* mod 65536 */
+
+		p_ptr->csp += (s16b)(new_mana);
 		p_ptr->csp_frac = (u16b)(new_mana_frac);
-	}
 
-	/* Must set frac to zero even if equal */
-	if (p_ptr->csp >= p_ptr->msp)
+		/* check for overflow */
+		if ((p_ptr->csp < 0) && (old_csp > 0))
+		{
+			p_ptr->csp = MAX_SHORT;
+		}
+
+		/* Must set frac to zero even if equal */
+		if (p_ptr->csp >= p_ptr->msp)
+		{
+			p_ptr->csp = p_ptr->msp;
+			p_ptr->csp_frac = 0;
+		}
+	}
+	else if (percent < 0)
 	{
-		p_ptr->csp = p_ptr->msp;
-		p_ptr->csp_frac = 0;
+		s32b dec_mana = ((long)p_ptr->msp) * (0 - percent) + PY_REGEN_MNBASE;
+
+		new_mana = (dec_mana >> 16);	/* div 65536 */
+		new_mana_frac = (dec_mana & 0xFFFF);	/* mod 65536 */
+
+		/* Check underflow */
+		if (new_mana_frac > p_ptr->csp_frac)
+		{
+			new_mana++;
+			p_ptr->csp_frac = 0;
+		}
+		else
+			p_ptr->csp_frac -= (u16b)(new_mana_frac);
+
+		p_ptr->csp -= (s16b)(new_mana);
+
+		/* Check underflow */
+		if (p_ptr->csp < 0)
+		{
+			p_ptr->csp = 0;
+			p_ptr->csp_frac = 0;
+		}
 	}
 
 	/* Redraw mana */
@@ -1929,62 +1951,47 @@ msg_print("ޤˤʢǵ䤷Ƥޤä");
 		regen_amount = regen_amount * 2;
 	}
 
-	if (total_friends)
-	{
-#ifdef TRACK_FRIENDS
-		if (wizard)
-#ifdef JP
-msg_format("ͧŪ󥹥: %d ", total_friends);
-#else
-			msg_format("Total friends: %d.", total_friends);
-#endif
-
-#endif /* TRACK_FRIENDS */
-
-		if (total_friends > 1 + (p_ptr->lev / cp_ptr->pet_upkeep_div))
-		{
-			upkeep_factor = total_friend_levels;
-
-			if (upkeep_factor > 95) upkeep_factor = 95;
-			else if (upkeep_factor < 5) upkeep_factor = 5;
-
-#ifdef TRACK_FRIENDS
-			if (wizard)
-#ifdef JP
-msg_format("٥ %d, ݻͣ %d", total_friend_levels, upkeep_factor);
-#else
-			msg_format("Levels %d, upkeep %d", total_friend_levels, upkeep_factor);
-#endif
-
-#endif /* TRACK_FRIENDS */
-
-		}
-	}
+	upkeep_factor = calculate_upkeep();
 
 	/* Regenerate the mana */
-	if (p_ptr->csp < p_ptr->msp)
+	if ((p_ptr->csp < p_ptr->msp) || (upkeep_factor != 0))
 	{
-		if (upkeep_factor)
-		{
-			s16b upkeep_regen = (((100 - upkeep_factor) * regen_amount) / 100);
-			regenmana(upkeep_regen);
+		s32b upkeep_regen = (((100 - upkeep_factor) * regen_amount) / 100);
+		regenmana(upkeep_regen);
 
 #ifdef TRACK_FRIENDS
-			if (wizard)
-			{
-#ifdef JP
-msg_format("ͣв: %d/%d", upkeep_regen, regen_amount);
-#else
-				msg_format("Regen: %d/%d", upkeep_regen, regen_amount);
-#endif
-
-			}
-#endif /* TRACK_FRIENDS */
-
-		}
-		else
+		if (wizard)
 		{
-			regenmana(regen_amount);
+#ifdef JP
+			msg_format("ͣв: %d/%d", upkeep_regen, regen_amount);
+#else
+			msg_format("Regen: %d/%d", upkeep_regen, regen_amount);
+#endif
+		}
+#endif /* TRACK_FRIENDS */
+	}
+
+	/* Empty mana becaused of controling too many pets */
+	if ((p_ptr->csp == 0) && (p_ptr->csp_frac == 0))
+	{
+		while (upkeep_factor > 100)
+		{
+#ifdef JP
+			msg_print("ʤ¿ΥڥåȤǤʤ");
+#else
+			msg_print("Too many pets to control at once!");
+#endif
+			msg_print(NULL);
+			do_cmd_pet_dismiss();
+
+			upkeep_factor = calculate_upkeep();
+
+#ifdef JP
+			msg_format("ݻͣФ %d%%", upkeep_factor);
+#else
+			msg_format("Upkeep: %d%% mana.", upkeep_factor);
+#endif
+			msg_print(NULL);
 		}
 	}
 
@@ -2187,6 +2194,24 @@ msg_format("ͣв: %d/%d", upkeep_regen, regen_amount);
 		(void)set_tim_radar(p_ptr->tim_radar - 1);
 	}
 
+	/* Magic: Revenge Sentence */
+	if (p_ptr->tim_sentence)
+	{
+		u16b flg = (PROJECT_GRID | PROJECT_ITEM | PROJECT_JUMP | PROJECT_KILL);
+		p_ptr->tim_sentence--;
+
+		if (!p_ptr->tim_sentence)
+		{
+#ifdef JP
+			msg_print("");
+#else
+			msg_print("Revenge, revenge");
+#endif
+			(void)project(0, 2, p_ptr->rvs_y, p_ptr->rvs_x, p_ptr->rvs_d, GF_MISSILE, flg);
+
+			p_ptr->redraw |= (PR_EXTRA);
+		}
+	}
 
 	/*** Poison and Stun and Cut ***/
 
@@ -2487,6 +2512,12 @@ else if (get_check("ƥݡȤޤ"))
 			/* Recharge */
 			o_ptr->timeout--;
 
+			/* Device-users can activate objects again quickly */
+			if (p_ptr->pclass == CLASS_DEVICE_USER)
+			{
+				if (o_ptr->timeout > 0) o_ptr->timeout--;
+			}
+
 			/* Notice changes */
 			if (!o_ptr->timeout)
 			{
@@ -2861,6 +2892,10 @@ static void process_command(void)
 	/* Reset current message line */
 	p_ptr->now_message = 0;
 
+	/* Sniper */
+	if ((p_ptr->pclass == CLASS_SNIPER) && (p_ptr->concent))
+		reset_concent = TRUE;
+
 	/* Parse the command */
 	switch (command_cmd)
 	{
@@ -3193,6 +3228,8 @@ msg_print("ɥ⡼");
 				do_cmd_mind_browse();
 			else if (p_ptr->pclass == CLASS_ELEMENTALIST)
 				do_cmd_elem_browse();
+			else if (p_ptr->pclass == CLASS_SNIPER)
+				do_cmd_snipe_browse();
 			else do_cmd_browse();
 			break;
 		}
@@ -3202,8 +3239,13 @@ msg_print("ɥ⡼");
 		{
 			if (!p_ptr->inside_arena)
 			{
+				if (p_ptr->pclass == CLASS_DEVICE_USER)
+				{
+					do_cmd_use_device();
+				}
+
 				/* -KMW- */
-				if (p_ptr->anti_magic)
+				else if (p_ptr->anti_magic)
 				{
 #ifdef JP
 					cptr which_power = "ˡ";
@@ -3236,6 +3278,10 @@ msg_print("ɥ⡼");
 						do_cmd_mindcraft();
 					else if (p_ptr->pclass == CLASS_ELEMENTALIST)
 						do_cmd_element();
+					else if (p_ptr->pclass == CLASS_SNIPER)
+						do_cmd_snipe();
+					else if (p_ptr->pclass == CLASS_SNATCHER)
+						do_cmd_monster_spell();
 					else
 						do_cmd_cast();
 				}
@@ -3311,7 +3357,7 @@ msg_print("꡼ʤˡۼ");
 		case 'f':
 		{
 			if (!p_ptr->inside_arena)
-				do_cmd_fire();
+				(void)do_cmd_fire();
 			else
 			{
 #ifdef JP
@@ -3604,7 +3650,11 @@ msg_print("꡼ʤˡۼ");
 		/* Show previous message */
 		case KTRL('O'):
 		{
+#ifdef TRAVEL
+			do_cmd_travel();
+#else
 			do_cmd_message_one();
+#endif
 			break;
 		}
 
@@ -3661,7 +3711,7 @@ msg_print("꡼ʤˡۼ");
 			do_cmd_suicide();
 			break;
 		}
-
+		
 		/* Check artifacts, uniques, objects */
 		case '~':
 		case '|':
@@ -3684,6 +3734,12 @@ msg_print("꡼ʤˡۼ");
 			break;
 		}
 
+		case ']':
+		{
+			prepare_movie_hooks();
+			break;
+		}
+
 		/* Hack -- Unknown command */
 		default:
 		{
@@ -3918,6 +3974,15 @@ static void process_player(void)
 			run_step(0);
 		}
 
+#ifdef TRAVEL
+		/* Traveling */
+		else if (travel.run)
+		{
+			/* Take a step */
+			travel_step();
+		}
+#endif
+
 		/* Repeated command */
 		else if (command_rep)
 		{
@@ -4075,7 +4140,12 @@ static void process_player(void)
 		if (p_ptr->leaving) break;
 		
 		/* Used up energy for this turn */
-		if (energy_use) break;
+		if (energy_use) 
+		{
+			if (reset_concent) reset_concentration(TRUE);
+			if (p_ptr->keep_magic) upkeep_magic_spell();
+			break;
+		}
 	}
 }
 
@@ -4128,21 +4198,13 @@ static void process_energy(void)
 	}
 
 	/* Can the player move? */
-	if (!ironman_hengband)
+	while (p_ptr->energy_need <= 0 && !p_ptr->leaving)
 	{
-		while (p_ptr->energy_need <= 0 && !p_ptr->leaving)
-		{
-			/* process monster with even more energy first */
-			process_monsters(p_ptr->energy_need - 1);
+		/* process monster with even more energy first */
+		process_monsters(p_ptr->energy_need - 1);
 
-			/* Process the player while still alive */
-			if (!p_ptr->leaving) process_player();
-		}
-	}
-	else /* ironman_hengband */
-	{
-		if (p_ptr->energy_need <= 0 && !p_ptr->leaving)
-			process_player();
+		/* Process the player while still alive */
+		if (!p_ptr->leaving) process_player();
 	}
 }
 
@@ -4444,6 +4506,20 @@ static void dungeon(void)
 		/* Un-mark the quest monster */
 		r_info[quest[quest_num].r_idx].flags1 &= ~RF1_QUESTOR;
 	}
+
+	/* Magic: Give up your revenge */
+	if (p_ptr->tim_sentence)
+	{
+		p_ptr->tim_sentence = 0;
+		p_ptr->rvs_x = 0;
+		p_ptr->rvs_y = 0;
+		p_ptr->rvs_d = 0;
+#ifdef JP
+		msg_print("򤢤᤿");
+#else
+		msg_print("Give up your revenge.");
+#endif
+	}
 }
 
 
@@ -4542,7 +4618,7 @@ void play_game(bool new_game)
 	/* Hack -- Turn off the cursor */
 	(void)Term_set_cursor(0);
 
-#ifdef CHUUKEI
+	/* for chuukei */
 	if (chuukei_client)
 	{
 		reset_visuals();
@@ -4553,7 +4629,13 @@ void play_game(bool new_game)
 	{
 		prepare_chuukei_hooks();
 	}
-#endif
+
+	if (browsing_movie)
+	{
+		reset_visuals();
+		browse_movie();
+		return;
+	}
 
 	/* Make sure main term is active */
 	Term_activate(angband_term[0]);
@@ -4706,19 +4788,11 @@ void play_game(bool new_game)
 	if (!dun_level && !p_ptr->inside_quest)
 	{
 		/* Init the wilderness */
-#ifdef JP
-		process_dungeon_file("w_info_j.txt", 0, 0, max_wild_y, max_wild_x);
-#else
 		process_dungeon_file("w_info.txt", 0, 0, max_wild_y, max_wild_x);
-#endif
 
 		/* Init the town */
 		init_flags = INIT_ONLY_BUILDINGS;
-#ifdef JP
-		process_dungeon_file("t_info_j.txt", 0, 0, MAX_HGT, MAX_WID);
-#else
 		process_dungeon_file("t_info.txt", 0, 0, MAX_HGT, MAX_WID);
-#endif
 	}
 
 	/* Initialize vault info */
