@@ -111,6 +111,9 @@ msg_print("ʤϺ𤷤");
 
 			/* Sniper */
 			if (p_ptr->concent) reset_concentration(TRUE);
+
+			/* Magic */
+			if (p_ptr->keep_magic) stop_magic_spell_all();
 		}
 	}
 
@@ -291,6 +294,12 @@ msg_print("Τ㤷Ƥޤä");
 #endif
 
 			notice = TRUE;
+
+			/* Sniper */
+			if (p_ptr->concent) reset_concentration(TRUE);
+
+			/* Magic */
+			if (p_ptr->keep_magic) stop_magic_spell_all();
 		}
 	}
 
@@ -735,7 +744,7 @@ bool set_blessed(int v)
 	/* Open */
 	if (v)
 	{
-		if (!p_ptr->blessed)
+		if ((!p_ptr->blessed) && (!is_keeping_spell(MS_BLESSING)))
 		{
 #ifdef JP
 msg_print("ʵʬˤʤä");
@@ -750,7 +759,7 @@ msg_print("ʵʬˤʤä");
 	/* Shut */
 	else
 	{
-		if (p_ptr->blessed)
+		if ((p_ptr->blessed) && (!is_keeping_spell(MS_BLESSING)))
 		{
 #ifdef JP
 msg_print("ʵʬä");
@@ -864,7 +873,7 @@ bool set_shero(int v)
 	/* Open */
 	if (v)
 	{
-		if (!p_ptr->shero)
+		if ((!p_ptr->shero) && (!is_keeping_spell(MS_BERSERK)))
 		{
 #ifdef JP
 msg_print("٤ޥˤʤä롪");
@@ -879,7 +888,7 @@ msg_print("٤ޥˤʤä롪");
 	/* Shut */
 	else
 	{
-		if (p_ptr->shero)
+		if ((p_ptr->shero) && (!is_keeping_spell(MS_BERSERK)))
 		{
 #ifdef JP
 msg_print("ڤʵä");
@@ -2109,6 +2118,9 @@ msg_print("褦Ƭˤ롣");
 
 		/* Sniper */
 		if (p_ptr->concent) reset_concentration(TRUE);
+
+		/* Magic */
+		if (p_ptr->keep_magic) stop_magic_spell_all();
 	}
 
 	/* Decrease cut */
@@ -3507,7 +3519,7 @@ take_hit(damroll(randint1(10), p_ptr->lev), "̿ŪѰ");
  * the game when he dies, since the "You die." message is shown before
  * setting the player to "dead".
  */
-void take_hit(int damage, cptr hit_from)
+int take_hit(int damage, cptr hit_from)
 {
 	int old_chp = p_ptr->chp;
 	bool pen_invuln = FALSE;
@@ -3515,7 +3527,7 @@ void take_hit(int damage, cptr hit_from)
 	int warning = (p_ptr->mhp * hitpoint_warn / 10);
 
 	/* Paranoia */
-	if (death) return;
+	if (death) return 0;
 
 	/* Disturb */
 	disturb(1, 0);
@@ -3529,7 +3541,7 @@ void take_hit(int damage, cptr hit_from)
 		}
 		else
 		{
-			return;
+			return 0;
 		}
 	}
 
@@ -3622,7 +3634,7 @@ void take_hit(int damage, cptr hit_from)
 		}
 
 		/* Dead */
-		return;
+		return (damage);
 	}
 
 	/* Hitpoint warning */
@@ -3644,6 +3656,8 @@ void take_hit(int damage, cptr hit_from)
 		flush();
 		msg_print(NULL);
 	}
+
+	return (damage);
 }
 
 
@@ -3658,12 +3672,24 @@ void gain_exp(s32b amount)
 	/* Snatchers can not gain experieces which are over maximum level of snatched body */
 	if (p_ptr->pclass == CLASS_SNATCHER)
 	{
-		if (p_ptr->exp > (player_exp[SN_MAX_LEVEL - 1] * p_ptr->expfact / 100L - 1L))
-			p_ptr->exp = (player_exp[SN_MAX_LEVEL - 1] * p_ptr->expfact / 100L - 1L);
+		/* Muximum experiences which Snatchers can gain */
+		int ex = (player_exp[SN_MAX_LEVEL - 1] * p_ptr->expfact / 100L - 1L);
+
+		if (p_ptr->exp > ex) p_ptr->exp = ex;
+
+		/* Slowly recover from experience drainage */
+		if (p_ptr->exp < p_ptr->max_exp)
+		{
+			int d = (ex - p_ptr->max_exp);
+
+			/* Gain max experience (20%) (was 10%) */
+			if (d > amount / 5) p_ptr->max_exp += amount / 5;
+			else if (d > 0) p_ptr->max_exp = ex;
+		}
 	}
 
 	/* Slowly recover from experience drainage */
-	if (p_ptr->exp < p_ptr->max_exp)
+	else if (p_ptr->exp < p_ptr->max_exp)
 	{
 		/* Gain max experience (20%) (was 10%) */
 		p_ptr->max_exp += amount / 5;
