@@ -64,6 +64,7 @@ static bool object_easy_know(int i)
 		case TV_LIFE_BOOK:
 		case TV_SORCERY_BOOK:
 		case TV_MUSOU_BOOK:
+		case TV_MAGIC_BOOK:
 		{
 			return (TRUE);
 		}
@@ -522,6 +523,7 @@ char *object_desc_kosuu(char *t, object_type *o_ptr)
 		case TV_LIFE_BOOK:
 		case TV_SORCERY_BOOK:
 		case TV_MUSOU_BOOK:
+		case TV_MAGIC_BOOK:
 		{
 			t = object_desc_str(t, "");
 			break;
@@ -822,7 +824,6 @@ static char *get_ability_abbreviation(char *ptr, object_type *o_ptr, bool kanji,
 	if (!all)
 	{
 		object_kind *k_ptr = &k_info[o_ptr->k_idx];
-		int j;
 
 		/* Base object */
 		flag[0] &= ~k_ptr->flags1;
@@ -1412,6 +1413,20 @@ void object_desc(char *buf, object_type *o_ptr, u32b mode)
 			break;
 		}
 
+		case TV_MAGIC_BOOK:
+		{
+#ifdef JP
+			basenm = "Ѥˡ%";
+#else
+			if (mp_ptr->spell_type == ST_PRAYER)
+				basenm = "& Book~ of Black Magic %";
+			else
+				basenm = "& Magic Spellbook~ %";
+#endif
+
+			break;
+		}
+
 		/* Hack -- Gold/Gems */
 		case TV_GOLD:
 		{
