@@ -199,8 +199,6 @@ bool make_attack_normal(int m_idx)
 
 	object_type *o_ptr;
 
-	object_kind *k_ptr;
-
 	char o_name[MAX_NLEN];
 
 	char m_name[80];
@@ -217,6 +215,9 @@ bool make_attack_normal(int m_idx)
 	int omit_mname = 0;
 #endif
 
+	/* Magic: for eye-for-an-eye */
+	int get_damage = 0;
+
 	/* Not allowed to attack */
 	if (r_ptr->flags1 & (RF1_NEVER_BLOW)) return (FALSE);
 
@@ -236,7 +237,6 @@ bool make_attack_normal(int m_idx)
 	/* Get the "died from" information (i.e. "a kobold") */
 	monster_desc(ddesc, m_ptr, 0x88);
 
-
 	/* Assume no blink */
 	blinked = FALSE;
 
@@ -246,7 +246,7 @@ bool make_attack_normal(int m_idx)
 		bool visible = FALSE;
 		bool obvious = FALSE;
 
-		int power = 0;
+		int power;
 		int damage = 0;
 
 		cptr act = NULL;
@@ -704,7 +704,7 @@ bool make_attack_normal(int m_idx)
 						damage -= (damage * ((ac < 150) ? ac : 150) / 250);
 
 						/* Take damage */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						break;
 					}
@@ -712,7 +712,7 @@ bool make_attack_normal(int m_idx)
 					case RBE_POISON:
 					{
 						/* Take some damage */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						/* Take "poison" effect */
 						if (!(p_ptr->resist_pois || p_ptr->oppose_pois))
@@ -732,7 +732,7 @@ bool make_attack_normal(int m_idx)
 					case RBE_UN_BONUS:
 					{
 						/* Take some damage */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						/* Allow complete resist */
 						if (!p_ptr->resist_disen)
@@ -750,7 +750,7 @@ bool make_attack_normal(int m_idx)
 					case RBE_UN_POWER:
 					{
 						/* Take some damage */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						/* Find an item */
 						for (k = 0; k < 10; k++)
@@ -772,14 +772,26 @@ bool make_attack_normal(int m_idx)
 								/* Calculate healed hitpoints */
 								int heal = rlev * o_ptr->pval;
 
+								if ((p_ptr->pclass == CLASS_DEVICE_USER) &&
+									(randint0(75 + r_ptr->level / 4) < p_ptr->lev * 2))
+								{
+#ifdef JP
+									msg_print("ͥ륮ζåɤ");
+#else
+									msg_print("You defended your devices from energy drain.");
+#endif
+									break;
+								}
+
 								if( o_ptr->tval == TV_STAFF)
 									heal *=  o_ptr->number;
+
 								/* Don't heal more than max hp */
 								heal = MIN(heal, m_ptr->maxhp - m_ptr->hp);
 
 								/* Message */
 #ifdef JP
-							msg_print("å饨ͥ륮ۤ줿");
+								msg_print("å饨ͥ륮ۤ줿");
 #else
 								msg_print("Energy drains from your pack!");
 #endif
@@ -814,7 +826,7 @@ bool make_attack_normal(int m_idx)
 					case RBE_EAT_GOLD:
 					{
 						/* Take some damage */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						/* Confused monsters cannot steal successfully. -LM-*/
 						if (m_ptr->confused) break;
@@ -893,7 +905,7 @@ bool make_attack_normal(int m_idx)
 					case RBE_EAT_ITEM:
 					{
 						/* Take some damage */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						/* Confused monsters cannot steal successfully. -LM-*/
 						if (m_ptr->confused) break;
@@ -937,7 +949,7 @@ bool make_attack_normal(int m_idx)
 							if (artifact_p(o_ptr) || o_ptr->art_name) continue;
 
 							/* Get a description */
-							object_desc(o_name, o_ptr, FALSE, 3);
+							object_desc(o_name, o_ptr, OD_OMIT_PREFIX);
 
 							/* Message */
 #ifdef JP
@@ -976,13 +988,12 @@ bool make_attack_normal(int m_idx)
 									 */
 									if ((o_ptr->tval == TV_ROD) || (o_ptr->tval == TV_WAND))
 									{
-										k_ptr = &k_info[o_ptr->k_idx];
 										j_ptr->pval = o_ptr->pval / o_ptr->number;
 										o_ptr->pval -= j_ptr->pval;
 									}
 
 									/* Forget mark */
-									j_ptr->marked = 0;
+									j_ptr->marked = OM_TOUCHED;
 
 									/* Memorize monster */
 									j_ptr->held_m_idx = m_idx;
@@ -1015,7 +1026,7 @@ bool make_attack_normal(int m_idx)
 					case RBE_EAT_FOOD:
 					{
 						/* Take some damage */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						/* Steal some food */
 						for (k = 0; k < 10; k++)
@@ -1033,7 +1044,7 @@ bool make_attack_normal(int m_idx)
 							if (o_ptr->tval != TV_FOOD) continue;
 
 							/* Get a description */
-							object_desc(o_name, o_ptr, FALSE, 0);
+							object_desc(o_name, o_ptr, OD_OMIT_PREFIX | OD_NAME_ONLY);
 
 							/* Message */
 #ifdef JP
@@ -1064,7 +1075,7 @@ bool make_attack_normal(int m_idx)
 					case RBE_EAT_LITE:
 					{
 						/* Take some damage */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						/* Access the lite */
 						o_ptr = &inventory[INVEN_LITE];
@@ -1109,7 +1120,7 @@ bool make_attack_normal(int m_idx)
 
 
 						/* Special damage */
-						acid_dam(damage, ddesc);
+						get_damage += acid_dam(damage, ddesc);
 
 						/* Learn about the player */
 						update_smart_learn(m_idx, DRS_ACID);
@@ -1131,7 +1142,7 @@ bool make_attack_normal(int m_idx)
 
 
 						/* Special damage */
-						elec_dam(damage, ddesc);
+						get_damage += elec_dam(damage, ddesc);
 
 						/* Learn about the player */
 						update_smart_learn(m_idx, DRS_ELEC);
@@ -1153,7 +1164,7 @@ bool make_attack_normal(int m_idx)
 
 
 						/* Special damage */
-						fire_dam(damage, ddesc);
+						get_damage += fire_dam(damage, ddesc);
 
 						/* Learn about the player */
 						update_smart_learn(m_idx, DRS_FIRE);
@@ -1175,7 +1186,7 @@ bool make_attack_normal(int m_idx)
 
 
 						/* Special damage */
-						cold_dam(damage, ddesc);
+						get_damage += cold_dam(damage, ddesc);
 
 						/* Learn about the player */
 						update_smart_learn(m_idx, DRS_COLD);
@@ -1186,7 +1197,7 @@ bool make_attack_normal(int m_idx)
 					case RBE_BLIND:
 					{
 						/* Take damage */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						/* Increase "blind" */
 						if (!p_ptr->resist_blind)
@@ -1210,7 +1221,7 @@ bool make_attack_normal(int m_idx)
 					case RBE_CONFUSE:
 					{
 						/* Take damage */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						/* Increase "confused" */
 						if (!p_ptr->resist_conf)
@@ -1230,7 +1241,7 @@ bool make_attack_normal(int m_idx)
 					case RBE_TERRIFY:
 					{
 						/* Take damage */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						/* Increase "afraid" */
 						if (p_ptr->resist_fear)
@@ -1273,7 +1284,7 @@ bool make_attack_normal(int m_idx)
 						if (p_ptr->paralyzed && (damage < 1)) damage = 1;
 
 						/* Take damage */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						/* Increase "paralyzed" */
 						if (p_ptr->free_act)
@@ -1313,7 +1324,7 @@ bool make_attack_normal(int m_idx)
 					case RBE_LOSE_STR:
 					{
 						/* Damage (physical) */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						/* Damage (stat) */
 						if (do_dec_stat(A_STR)) obvious = TRUE;
@@ -1324,7 +1335,7 @@ bool make_attack_normal(int m_idx)
 					case RBE_LOSE_INT:
 					{
 						/* Damage (physical) */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						/* Damage (stat) */
 						if (do_dec_stat(A_INT)) obvious = TRUE;
@@ -1335,7 +1346,7 @@ bool make_attack_normal(int m_idx)
 					case RBE_LOSE_WIS:
 					{
 						/* Damage (physical) */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						/* Damage (stat) */
 						if (do_dec_stat(A_WIS)) obvious = TRUE;
@@ -1346,7 +1357,7 @@ bool make_attack_normal(int m_idx)
 					case RBE_LOSE_DEX:
 					{
 						/* Damage (physical) */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						/* Damage (stat) */
 						if (do_dec_stat(A_DEX)) obvious = TRUE;
@@ -1357,7 +1368,7 @@ bool make_attack_normal(int m_idx)
 					case RBE_LOSE_CON:
 					{
 						/* Damage (physical) */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						/* Damage (stat) */
 						if (do_dec_stat(A_CON)) obvious = TRUE;
@@ -1368,7 +1379,7 @@ bool make_attack_normal(int m_idx)
 					case RBE_LOSE_CHR:
 					{
 						/* Damage (physical) */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						/* Damage (stat) */
 						if (do_dec_stat(A_CHR)) obvious = TRUE;
@@ -1379,7 +1390,7 @@ bool make_attack_normal(int m_idx)
 					case RBE_LOSE_ALL:
 					{
 						/* Damage (physical) */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						/* Damage (stats) */
 						if (do_dec_stat(A_STR)) obvious = TRUE;
@@ -1401,7 +1412,7 @@ bool make_attack_normal(int m_idx)
 						damage -= (damage * ((ac < 150) ? ac : 150) / 250);
 
 						/* Take damage */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						/* Radius 8 earthquake centered at the monster */
 						if (damage > 23)
@@ -1418,7 +1429,7 @@ bool make_attack_normal(int m_idx)
 						obvious = TRUE;
 
 						/* Take damage */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						if (p_ptr->hold_life && (randint0(100) < 95))
 						{
@@ -1462,7 +1473,7 @@ bool make_attack_normal(int m_idx)
 						obvious = TRUE;
 
 						/* Take damage */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						if (p_ptr->hold_life && (randint0(100) < 90))
 						{
@@ -1506,7 +1517,7 @@ bool make_attack_normal(int m_idx)
 						obvious = TRUE;
 
 						/* Take damage */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						if (p_ptr->hold_life && (randint0(100) < 75))
 						{
@@ -1550,7 +1561,7 @@ bool make_attack_normal(int m_idx)
 						obvious = TRUE;
 
 						/* Take damage */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						if (p_ptr->hold_life && (randint0(100) < 50))
 						{
@@ -1591,7 +1602,7 @@ bool make_attack_normal(int m_idx)
 					case RBE_DISEASE:
 					{
 						/* Take some damage */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						/* Take "poison" effect */
 						if (!(p_ptr->resist_pois || p_ptr->oppose_pois))
@@ -1691,7 +1702,7 @@ bool make_attack_normal(int m_idx)
 								break;
 							}
 						}
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						break;
 					}
@@ -1701,7 +1712,7 @@ bool make_attack_normal(int m_idx)
 						obvious = TRUE;
 
 						/* Take damage */
-						take_hit(damage, ddesc);
+						get_damage += take_hit(damage, ddesc);
 
 						if (p_ptr->hold_life && (randint0(100) < 50))
 						{
@@ -1791,7 +1802,7 @@ msg_format("%sϤ褦", m_name);
 			/* Handle cut */
 			if (do_cut)
 			{
-				int k = 0;
+				int k;
 
 				/* Critical hit (zero if non-critical) */
 				tmp = monster_critical(d_dice, d_side, damage);
@@ -1816,7 +1827,7 @@ msg_format("%sϤ褦", m_name);
 			/* Handle stun */
 			if (do_stun)
 			{
-				int k = 0;
+				int k;
 
 				/* Critical hit (zero if non-critical) */
 				tmp = monster_critical(d_dice, d_side, damage);
@@ -2008,17 +2019,45 @@ msg_format("%sϤ褦", m_name);
 		}
 	}
 
+	/* Magic: Eye for an Eye */
+	if (is_keeping_spell(MS_EYE_FOR_EYE) && (get_damage > 0) && (m_idx > 0))
+	{
+#ifdef JP
+		msg_print("Ϥ᡼֤");
+#else
+		msg_print("You return your damage.");
+#endif
+		mon_take_hit(m_idx, get_damage, &fear, NULL);
+	}
+
+	/* Magic: Revenge Sentence */
+	if ((p_ptr->tim_sentence) && (get_damage > 0))
+	{
+		p_ptr->rvs_x = m_ptr->fx;
+		p_ptr->rvs_y = m_ptr->fy;
+		p_ptr->rvs_d += get_damage;
+	}
 
 	/* Blink away */
 	if (blinked)
 	{
+		if (teleport_barrier(m_idx))
+		{
 #ifdef JP
-		msg_print("ťϾФäƨ");
+			msg_print("ťϾФäƨ...褦ȤХꥢɤ줿");
 #else
-		msg_print("The thief flees laughing!");
+			msg_print("The thief flees laughing...? But magic barrier obstructs it.");
 #endif
-
-		teleport_away(m_idx, MAX_SIGHT * 2 + 5);
+		}
+		else
+		{
+#ifdef JP
+			msg_print("ťϾФäƨ");
+#else
+			msg_print("The thief flees laughing!");
+#endif
+			teleport_away(m_idx, MAX_SIGHT * 2 + 5);
+		}
 	}
 
 
