@@ -271,12 +271,7 @@ void mon_take_hit_mon(int m_idx, int dam, bool *fear, cptr note, int who)
 				/* Death by special attack */
 				else if (note)
 				{
-#ifdef JP
 					msg_format("%^s%s", m_name, note);
-#else
-					msg_format("%^s%s", m_name, note);
-#endif
-
 				}
 				/* Death by normal attack -- nonliving monster */
 				else if (!monster_living(r_ptr))
@@ -2141,21 +2136,38 @@ mon_take_hit_mon(m_idx, m_ptr->hp + 1, &fear, "ȯʴˤʤä", m_idx);
 	/* Blink away */
 	if (blinked)
 	{
-		if (see_m)
+		if (teleport_barrier(m_idx))
 		{
+			if (see_m)
+			{
 #ifdef JP
-msg_print("ťϾФäƨ");
+				msg_print("ťϾФäƨ...褦ȤХꥢɤ줿");
 #else
-			msg_print("The thief flees laughing!");
+				msg_print("The thief flees laughing...? But magic barrier obstructs it.");
 #endif
-
+			}
+			else if (known)
+			{
+				mon_fight = TRUE;
+			}
 		}
-		else if (known)
+		else
 		{
-			mon_fight = TRUE;
-		}
+			if (see_m)
+			{
+	#ifdef JP
+				msg_print("ťϾФäƨ");
+	#else
+				msg_print("The thief flees laughing!");
+	#endif
+			}
+			else if (known)
+			{
+				mon_fight = TRUE;
+			}
 
-		teleport_away(m_idx, MAX_SIGHT * 2 + 5);
+			teleport_away(m_idx, MAX_SIGHT * 2 + 5);
+		}
 	}
 
 	return TRUE;
@@ -2589,6 +2601,9 @@ static void process_monster(int m_idx)
 			}
 		}
 
+		/* Magic: Anti multiply barrier obstructs multuplying of monsters */
+		if (multiply_barrier(m_idx)) k = 8;
+
 		/* Hack -- multiply slower in crowded areas */
 		if ((k < 4) && (!k || !randint0(k * MON_MULT_ADJ)))
 		{
