@@ -1466,6 +1466,16 @@ void update_mon(int m_idx, bool full)
 			}
 		}
 
+		/* Magic */
+		if (is_keeping_spell(MS_DETECT_EVIL))
+		{
+			if (r_ptr->flags3 & (RF3_EVIL))
+			{
+				r_ptr->r_flags3 |= (RF3_EVIL);
+				flag = TRUE;
+			}
+		}
+
 		/* Normal line of sight, and not blind */
 		if (player_has_los_bold(fy, fx) && !p_ptr->blind)
 		{
