@@ -816,6 +816,7 @@ static bool projectable_to_next(int y1, int x1, int *yy, int *xx)
  */
 bool make_attack_spell(int m_idx)
 {
+	int             i;
 	int             k, chance, thrown_spell, rlev, failrate;
 	u32b            f4, f5, f6;
 	monster_type    *m_ptr = &m_list[m_idx];
@@ -994,7 +995,11 @@ bool make_attack_spell(int m_idx)
 	/* Hack -- Get the "died from" name */
 	monster_desc(ddesc, m_ptr, 0x88);
 
-	thrown_spell = choose_attack_spell(m_idx, f4, f5, f6);
+	for (i = 0; i < 10; i++)
+	{
+		thrown_spell = choose_attack_spell(m_idx, f4, f5, f6);
+		if (thrown_spell) break;
+	}
 
 	/* Abort if no spell was chosen */
 	if (!thrown_spell) return (FALSE);
@@ -1017,6 +1022,17 @@ bool make_attack_spell(int m_idx)
 		return (TRUE);
 	}
 
+	/* Magic: Anti Magic Barrier */
+	if ((thrown_spell >= 128) && magic_barrier(m_idx))
+	{
+#ifdef JP
+		msg_format("ȿˡХꥢ%^sμʸ򤫤ä", m_name);
+#else
+		msg_format("Anti magic barrier cancels the spell which %^s casts.");
+#endif
+		return (TRUE);
+	}
+
 	/* Cast the spell. */
 	switch (thrown_spell)
 	{
@@ -2479,7 +2495,7 @@ bool make_attack_spell(int m_idx)
 				else msg_format("%^sϴ˼ä褦", m_name);
 #else
 				if (seen) msg_format("%^s looks completely healed!", m_name);
-				elsemsg_format("%^s sounds completely healed!", m_name);
+				else msg_format("%^s sounds completely healed!", m_name);
 #endif
 			}
 
@@ -2544,13 +2560,24 @@ bool make_attack_spell(int m_idx)
 		case 160+4:
 		{
 			disturb(1, 0);
+			if (teleport_barrier(m_idx))
+			{
 #ifdef JP
-			msg_format("%^sֻ˾ä", m_name);
+				msg_format("ˡΥХꥢ%^sΥƥݡȤ⤷", m_name);
 #else
-			msg_format("%^s blinks away.", m_name);
+				msg_format("Magic barrier obstructs teleporting of %^s.", m_name);
 #endif
-			teleport_away(m_idx, 10);
-			p_ptr->update |= (PU_MONSTERS | PU_MON_LITE);
+			}
+			else
+			{
+#ifdef JP
+				msg_format("%^sֻ˾ä", m_name);
+#else
+				msg_format("%^s blinks away.", m_name);
+#endif
+				teleport_away(m_idx, 10);
+				p_ptr->update |= (PU_MONSTERS | PU_MON_LITE);
+			}
 			break;
 		}
 
@@ -2558,12 +2585,23 @@ bool make_attack_spell(int m_idx)
 		case 160+5:
 		{
 			disturb(1, 0);
+			if (teleport_barrier(m_idx))
+			{
 #ifdef JP
-			msg_format("%^sƥݡȤ", m_name);
+				msg_format("ˡΥХꥢ%^sΥƥݡȤ⤷", m_name);
 #else
-			msg_format("%^s teleports away.", m_name);
+				msg_format("Magic barrier obstructs teleporting of %^s.", m_name);
 #endif
-			teleport_away(m_idx, MAX_SIGHT * 2 + 5);
+			}
+			else
+			{
+#ifdef JP
+				msg_format("%^sƥݡȤ", m_name);
+#else
+				msg_format("%^s teleports away.", m_name);
+#endif
+				teleport_away(m_idx, MAX_SIGHT * 2 + 5);
+			}
 			break;
 		}
 
