@@ -2414,32 +2414,60 @@ bool monst_spell_monst(int m_idx)
 			/* RF6_BLINK */
 			case 160+4:
 			{
-				if (see_m)
+				if (teleport_barrier(m_idx))
 				{
+					if (see_m)
+					{
 #ifdef JP
-					msg_format("%^sֻ˾ä", m_name);
+						msg_format("ˡΥХꥢ%^sΥƥݡȤ⤷", m_name);
 #else
-					msg_format("%^s blinks away.", m_name);
+						msg_format("Magic barrier obstructs teleporting of %^s.", m_name);
 #endif
+					}
+				}
+				else
+				{
+					if (see_m)
+					{
+#ifdef JP
+						msg_format("%^sֻ˾ä", m_name);
+#else
+						msg_format("%^s blinks away.", m_name);
+#endif
+					}
+					teleport_away(m_idx, 10);
 				}
 
-				teleport_away(m_idx, 10);
 				break;
 			}
 
 			/* RF6_TPORT */
 			case 160+5:
 			{
-				if (see_m)
+				if (teleport_barrier(m_idx))
 				{
+					if (see_m)
+					{
 #ifdef JP
-					msg_format("%^sƥݡȤ", m_name);
+						msg_format("ˡΥХꥢ%^sΥƥݡȤ⤷", m_name);
 #else
-					msg_format("%^s teleports away.", m_name);
+						msg_format("Magic barrier obstructs teleporting of %^s.", m_name);
 #endif
+					}
 				}
+				else
+				{
+					if (see_m)
+					{
+#ifdef JP
+						msg_format("%^sƥݡȤ", m_name);
+#else
+						msg_format("%^s teleports away.", m_name);
+#endif
+					}
 
-				teleport_away(m_idx, MAX_SIGHT * 2 + 5);
+					teleport_away(m_idx, MAX_SIGHT * 2 + 5);
+				}
 				break;
 			}
 
