@@ -735,6 +735,95 @@ static bool do_cmd_arrow_rain(void)
 	return (TRUE);
 }
 
+static bool item_tester_hook_devices(object_type *o_ptr)
+{
+	switch (o_ptr->tval)
+	{
+		case TV_WAND:
+		case TV_STAFF:
+		case TV_ROD:
+		{
+			return (bool) (!object_known_p(o_ptr));
+		}
+	}
+	return (FALSE);
+}
+
+static bool do_cmd_identify_devices(void)
+{
+	int item;
+	cptr q, s;
+	object_type *o_ptr;
+	char o_name[MAX_NLEN];
+	bool old_known;
+
+	item_tester_hook = item_tester_hook_devices;
+
+	/* Get an item */
+#ifdef JP
+	q = "ɤΥƥꤷޤ? ";
+	s = "Ǥ륢ƥबʤ";
+#else
+	q = "Identify which item? ";
+	s = "You have nothing to identify.";
+#endif
+
+	if (!get_item(&item, q, s, (USE_EQUIP | USE_INVEN | USE_FLOOR))) return (FALSE);
+
+	/* Access the item (if in the pack) */
+	if (item >= 0)
+	{
+		o_ptr = &inventory[item];
+	}
+	else
+	{
+		o_ptr = &o_list[0 - item];
+	}
+
+	/* Identify it */
+	old_known = identify_item(o_ptr);
+
+	/* Handle stuff */
+	handle_stuff();
+
+	/* Description */
+	object_desc(o_name, o_ptr, 0);
+
+	/* Describe */
+	if (item >= INVEN_WIELD)
+	{
+#ifdef JP
+		msg_format("%^s: %s(%c)",
+#else
+		msg_format("%^s: %s (%c).",
+#endif
+			   describe_use(item), o_name, index_to_label(item));
+	}
+	else if (item >= 0)
+	{
+#ifdef JP
+		msg_format("å: %s(%c)",
+#else
+		msg_format("In your pack: %s (%c).",
+#endif
+			   o_name, index_to_label(item));
+	}
+	else
+	{
+#ifdef JP
+		msg_format(": %s",
+#else
+		msg_format("On the ground: %s.",
+#endif
+			   o_name);
+	}
+
+	/* Auto-inscription/destroy */
+	autopick_alter_item(item, (bool)(destroy_identify && !old_known));
+
+	return (TRUE);
+}
+
 
 static bool cmd_racial_power_aux(s32b command)
 {
@@ -751,6 +840,11 @@ static bool cmd_racial_power_aux(s32b command)
 				if (!do_cmd_shield_bash()) return FALSE;
 				break;
 			case CLASS_MAGE:
+				if (p_ptr->realm1 == REALM_MAGIC)	/* Magic */
+				{
+					if (!stop_magic_spell()) return FALSE;
+					break;
+				}
 			case CLASS_PRIEST:
 			case CLASS_MINDCRAFTER:
 				if (!do_cmd_restore_mana()) return FALSE;
@@ -853,6 +947,16 @@ static bool cmd_racial_power_aux(s32b command)
 					teleport_player(5);
 				}
 				break;
+			case CLASS_DEVICE_USER:
+				if (command == -3)
+				{
+					if (!do_cmd_identify_devices()) return FALSE;
+				}
+				else if (command == -4)
+				{
+					if (!recharge_energy(p_ptr->lev * 5)) return FALSE;
+				}
+				break;
 		}
 	}
 	else switch (p_ptr->prace)
@@ -1061,6 +1165,7 @@ static bool cmd_racial_power_aux(s32b command)
 						case CLASS_WARRIOR_MAGE:
 						case CLASS_HIGH_MAGE:
 						case CLASS_ARCHAEOLOGIST:
+						case CLASS_DEVICE_USER:
 							if (randint1(3) == 1)
 							{
 								Type = GF_MANA;
@@ -1369,6 +1474,20 @@ void do_cmd_racial_power(void)
 			power_desc[num++].number = -3;
 			break;
 		case CLASS_MAGE:
+			if (p_ptr->realm1 == REALM_MAGIC)
+			{
+#ifdef JP
+				strcpy(power_desc[num].name, "ʸ");
+#else
+				strcpy(power_desc[num].name, "Stop spell");
+#endif
+				power_desc[num].level = 1;
+				power_desc[num].cost = 0;
+				power_desc[num].stat = m_info[p_ptr->pclass].spell_stat;
+				power_desc[num].fail = 0;
+				power_desc[num++].number = -3;
+				break;
+			}
 		case CLASS_PRIEST:
 		case CLASS_MINDCRAFTER:
 #ifdef JP
@@ -1576,6 +1695,28 @@ void do_cmd_racial_power(void)
 				power_desc[num++].number = -4;
 			}
 			break;
+		case CLASS_DEVICE_USER:
+#ifdef JP
+			strcpy(power_desc[num].name, "ˡƻ");
+#else
+			strcpy(power_desc[num].name, "Identify magical devices");
+#endif
+			power_desc[num].level = 5;
+			power_desc[num].cost = 5;
+			power_desc[num].stat = A_INT;
+			power_desc[num].fail = 5;
+			power_desc[num++].number = -3;
+#ifdef JP
+			strcpy(power_desc[num].name, format(" (ѥ %d)", p_ptr->lev * 5));
+#else
+			strcpy(power_desc[num].name, format("Identify magical devices (power %d)", p_ptr->lev * 5));
+#endif
+			power_desc[num].level = 7;
+			power_desc[num].cost = 10;
+			power_desc[num].stat = A_INT;
+			power_desc[num].fail = 10;
+			power_desc[num++].number = -4;
+			break;
 		default:
 #ifdef JP
 			strcpy(power_desc[num].name, "(ʤ)");
