@@ -1054,13 +1054,13 @@ static void wr_extra(void)
 	wr_s16b(p_ptr->tim_wraith);
 	wr_s16b(p_ptr->resist_magic);
 	wr_s16b(p_ptr->tim_radar);
+	wr_s16b(p_ptr->tim_sentence);	/* Magic */
 	wr_s16b(p_ptr->tim_xtra1);
 	wr_s16b(p_ptr->tim_xtra2);
 	wr_s16b(p_ptr->tim_xtra3);
 	wr_s16b(p_ptr->tim_xtra4);
 	wr_s16b(p_ptr->tim_xtra5);
 	wr_s16b(p_ptr->tim_xtra6);
-	wr_s16b(p_ptr->tim_xtra7);
 
 	wr_s16b(p_ptr->chaos_patron);
 	wr_u32b(p_ptr->muta);
@@ -1076,6 +1076,13 @@ static void wr_extra(void)
 	wr_byte(preserve_mode);
 	wr_byte(0);
 
+	/* Magic */
+	wr_s16b(p_ptr->keep_magic);
+	wr_u32b(p_ptr->keep_spells);
+	wr_s16b(p_ptr->rvs_x);
+	wr_s16b(p_ptr->rvs_y);
+	wr_s32b(p_ptr->rvs_d);
+
 	/* Future use */
 	for (i = 0; i < 12; i++) wr_u32b(0L);
 
