@@ -102,16 +102,12 @@ info[i++] = "ʤϿǤ롣(10 MP)";
 			break;
 		case RACE_GNOME:
 			if (plev > 4)
-			{
 #ifdef JP
-sprintf(Dummy, "ʤϰ %d ˥ƥݡȤǤ롣(%d MP)",
+info[i++] = "ʤϰ 10 ˥ƥݡȤǤ롣(5 MP)";
 #else
-				sprintf(Dummy, "You can teleport, range %d (cost %d).",
+				info[i++] = "You can teleport, range 10 (cost 5).";
 #endif
 
-				    (1 + plev), (5 + (plev / 5)));
-				info[i++] = Dummy;
-			}
 			break;
 		case RACE_HALF_ORC:
 			if (plev > 2)
@@ -651,7 +647,7 @@ info[i++] = "ʤΰ֤ϤҤ礦԰";
 #endif
 
 	}
-	if (p_ptr->blessed)
+	if ((p_ptr->blessed) || (is_keeping_spell(MS_BLESSING)))
 	{
 #ifdef JP
 info[i++] = "ʤϸ򴶤Ƥ롣";
@@ -669,7 +665,7 @@ info[i++] = "ʤϥҡʬ";
 #endif
 
 	}
-	if (p_ptr->shero)
+	if ((p_ptr->shero) || (is_keeping_spell(MS_BERSERK)))
 	{
 #ifdef JP
 info[i++] = "ʤƮ";
@@ -814,7 +810,7 @@ info[i++] = "ʤϿߤʤ";
 	if (p_ptr->telepathy)
 	{
 #ifdef JP
-info[i++] = "ʤĶǽϤäƤ롣";
+info[i++] = "ʤϥƥѥǽϤäƤ롣";
 #else
 		info[i++] = "You have ESP.";
 #endif
@@ -1287,6 +1283,14 @@ info[i++] = "ʤ̥ϤˤäƱƶƤ롣";
 
 	}
 
+	if (f1 & (TR1_MAGIC_MASTERY))
+	{
+#ifdef JP
+		info[i++] = "ʤˡƻǽϤˤäƱƶƤ롣";
+#else
+		info[i++] = "Your magic device is affected by your equipment.";
+#endif
+	}
 	if (f1 & (TR1_STEALTH))
 	{
 #ifdef JP
@@ -1593,320 +1597,6 @@ prt("[򲡤ȥޤ]", k, 13);
 }
 
 
-static int report_magics_aux(int dur)
-{
-	if (dur <= 5)
-	{
-		return 0;
-	}
-	else if (dur <= 10)
-	{
-		return 1;
-	}
-	else if (dur <= 20)
-	{
-		return 2;
-	}
-	else if (dur <= 50)
-	{
-		return 3;
-	}
-	else if (dur <= 100)
-	{
-		return 4;
-	}
-	else if (dur <= 200)
-	{
-		return 5;
-	}
-	else
-	{
-		return 6;
-	}
-}
-
-static cptr report_magic_durations[] =
-{
-#ifdef JP
-"û",
-"δ",
-"Ф餯δ",
-"¿Ĺ",
-"Ĺ",
-"Ĺ",
-"񤤤ۤĹ",
-"󥹥򹶷⤹ޤ"
-#else
-	"for a short time",
-	"for a little while",
-	"for a while",
-	"for a long while",
-	"for a long time",
-	"for a very long time",
-	"for an incredibly long time",
-	"until you hit a monster"
-#endif
-
-};
-
-
-/*
- * Report all currently active magical effects.
- */
-void report_magics(void)
-{
-	int     i = 0, j, k;
-	char    Dummy[80];
-	cptr    info[128];
-	int     info2[128];
-
-
-	if (p_ptr->blind)
-	{
-		info2[i]  = report_magics_aux(p_ptr->blind);
-#ifdef JP
-info[i++] = "ʤܤʤ";
-#else
-		info[i++] = "You cannot see";
-#endif
-
-	}
-	if (p_ptr->confused)
-	{
-		info2[i]  = report_magics_aux(p_ptr->confused);
-#ifdef JP
-info[i++] = "ʤϺ𤷤Ƥ롣";
-#else
-		info[i++] = "You are confused";
-#endif
-
-	}
-	if (p_ptr->afraid)
-	{
-		info2[i]  = report_magics_aux(p_ptr->afraid);
-#ifdef JP
-info[i++] = "ʤ϶ݤ˿Ƥ롣";
-#else
-		info[i++] = "You are terrified";
-#endif
-
-	}
-	if (p_ptr->poisoned)
-	{
-		info2[i]  = report_magics_aux(p_ptr->poisoned);
-#ifdef JP
-info[i++] = "ʤǤ˿Ƥ롣";
-#else
-		info[i++] = "You are poisoned";
-#endif
-
-	}
-	if (p_ptr->image)
-	{
-		info2[i]  = report_magics_aux(p_ptr->image);
-#ifdef JP
-info[i++] = "ʤϸФ򸫤Ƥ롣";
-#else
-		info[i++] = "You are hallucinating";
-#endif
-
-	}
-	if (p_ptr->blessed)
-	{
-		info2[i]  = report_magics_aux(p_ptr->blessed);
-#ifdef JP
-info[i++] = "ʤϸ򴶤Ƥ롣";
-#else
-		info[i++] = "You feel rightous";
-#endif
-
-	}
-	if (p_ptr->hero)
-	{
-		info2[i]  = report_magics_aux(p_ptr->hero);
-#ifdef JP
-info[i++] = "ʤϥҡʬ";
-#else
-		info[i++] = "You feel heroic";
-#endif
-
-	}
-	if (p_ptr->shero)
-	{
-		info2[i]  = report_magics_aux(p_ptr->shero);
-#ifdef JP
-info[i++] = "ʤƮ";
-#else
-		info[i++] = "You are in a battle rage";
-#endif
-
-	}
-	if (p_ptr->protevil)
-	{
-		info2[i]  = report_magics_aux(p_ptr->protevil);
-#ifdef JP
-info[i++] = "ʤϼٰʤ¸ߤƤ롣";
-#else
-		info[i++] = "You are protected from evil";
-#endif
-
-	}
-	if (p_ptr->shield)
-	{
-		info2[i]  = report_magics_aux(p_ptr->shield);
-#ifdef JP
-info[i++] = "ʤϿΥɤǼƤ롣";
-#else
-		info[i++] = "You are protected by a mystic shield";
-#endif
-
-	}
-	if (p_ptr->invuln)
-	{
-		info2[i]  = report_magics_aux(p_ptr->invuln);
-#ifdef JP
-info[i++] = "̵ŨǤ롣";
-#else
-		info[i++] = "You are invulnerable";
-#endif
-
-	}
-	if (p_ptr->wraith_form)
-	{
-		info2[i]  = report_magics_aux(p_ptr->wraith_form);
-#ifdef JP
-info[i++] = "ͩβǤ롣";
-#else
-		info[i++] = "You are incorporeal";
-#endif
-
-	}
-	if (p_ptr->confusing)
-	{
-		info2[i]  = 7;
-#ifdef JP
-info[i++] = "ʤμ֤Ƥ롣";
-#else
-		info[i++] = "Your hands are glowing dull red.";
-#endif
-
-	}
-	if (p_ptr->word_recall)
-	{
-		info2[i]  = report_magics_aux(p_ptr->word_recall);
-#ifdef JP
-info[i++] = "θ嵢Ԥξۤȯư롣";
-#else
-		info[i++] = "You are waiting to be recalled";
-#endif
-
-	}
-	if (p_ptr->oppose_acid)
-	{
-		info2[i]  = report_magics_aux(p_ptr->oppose_acid);
-#ifdef JP
-info[i++] = "ʤϻؤäƤ롣";
-#else
-		info[i++] = "You are resistant to acid";
-#endif
-
-	}
-	if (p_ptr->oppose_elec)
-	{
-		info2[i]  = report_magics_aux(p_ptr->oppose_elec);
-#ifdef JP
-info[i++] = "ʤŷؤäƤ롣";
-#else
-		info[i++] = "You are resistant to lightning";
-#endif
-
-	}
-	if (p_ptr->oppose_fire)
-	{
-		info2[i]  = report_magics_aux(p_ptr->oppose_fire);
-#ifdef JP
-info[i++] = "ʤϲФؤäƤ롣";
-#else
-		info[i++] = "You are resistant to fire";
-#endif
-
-	}
-	if (p_ptr->oppose_cold)
-	{
-		info2[i]  = report_magics_aux(p_ptr->oppose_cold);
-#ifdef JP
-info[i++] = "ʤ䵤ؤäƤ롣";
-#else
-		info[i++] = "You are resistant to cold";
-#endif
-
-	}
-	if (p_ptr->oppose_pois)
-	{
-		info2[i]  = report_magics_aux(p_ptr->oppose_pois);
-#ifdef JP
-info[i++] = "ʤǤؤäƤ롣";
-#else
-		info[i++] = "You are resistant to poison";
-#endif
-
-	}
-
-	/* Save the screen */
-	screen_save();
-
-	/* Erase the screen */
-	for (k = 1; k < 24; k++) prt("", k, 13);
-
-	/* Label the information */
-#ifdef JP
-prt("           ˡ        :", 1, 15);
-#else
-	prt("     Your Current Magic:", 1, 15);
-#endif
-
-
-	/* We will print on top of the map (column 13) */
-	for (k = 2, j = 0; j < i; j++)
-	{
-		/* Show the info */
-#ifdef JP
-sprintf(Dummy, "ʤ%s%s", info[j],
-#else
-		sprintf(Dummy, "%s %s.", info[j],
-#endif
-
-			report_magic_durations[info2[j]]);
-		prt(Dummy, k++, 15);
-
-		/* Every 20 entries (lines 2 to 21), start over */
-		if ((k == 22) && (j + 1 < i))
-		{
-#ifdef JP
-prt("-- ³ --", k, 15);
-#else
-			prt("-- more --", k, 15);
-#endif
-
-			inkey();
-			for (; k > 2; k--) prt("", k, 15);
-		}
-	}
-
-	/* Pause */
-#ifdef JP
-prt("[򲡤ȥޤ]", k, 13);
-#else
-	prt("[Press any key to continue]", k, 13);
-#endif
-
-	inkey();
-
-	/* Restore the screen */
-	screen_load();
-}
-
-
 /*
  * Detect all traps on current panel
  */
@@ -2328,6 +2018,7 @@ bool detect_objects_magic(int range)
 		    (tv == TV_LIFE_BOOK) ||
 			(tv == TV_SORCERY_BOOK) ||
 		    (tv == TV_MUSOU_BOOK) ||
+		    (tv == TV_MAGIC_BOOK) ||
 		    ((o_ptr->to_a > 0) || (o_ptr->to_h + o_ptr->to_d > 0)))
 		{
 			/* Memorize the item */
@@ -2574,7 +2265,7 @@ bool detect_monsters_string(int range, cptr Match)
 		if (distance(py, px, y, x) > range) continue;
 
 		/* Detect monsters with the same symbol */
-		if (strchr(Match, r_ptr->d_char))
+		if (my_strchr(Match, r_ptr->d_char))
 		{
 			/* Update monster recall window */
 			if (p_ptr->monster_race_idx == m_ptr->r_idx)
@@ -2946,7 +2637,7 @@ else if (sleep) msg_print("ǡʳ褦󤯤ʹ");
 /*
  * ironman_hengbandONλ񹳤Ǥͤɲápower0ʤԲġ
  */
-bool remove_monster(int power, int m_idx, char *s)
+bool remove_monster(int power, int m_idx, cptr s)
 {
 	int msec = delay_factor * delay_factor * delay_factor;
 	bool resist = FALSE;
@@ -3123,7 +2814,6 @@ bool mass_genocide(int power)
 	for (i = 1; i < m_max; i++)
 	{
 		monster_type    *m_ptr = &m_list[i];
-		monster_race    *r_ptr = &r_info[m_ptr->r_idx];
 
 		/* Paranoia -- Skip dead monsters */
 		if (!m_ptr->r_idx) continue;
@@ -3169,6 +2859,7 @@ bool probing(void)
 		/* Probe visible monsters */
 		if (m_ptr->ml)
 		{
+			char key;
 			char m_name[80];
 
 			/* Start the message */
@@ -3191,9 +2882,12 @@ bool probing(void)
 #else
 			sprintf(buf, "%s ... HP:%d/%d AC:%d speed:%s%d ", m_name, m_ptr->hp, m_ptr->maxhp, r_ptr->ac, (speed > 0) ? "+" : "", speed);
 #endif
-
 			buf[strlen(buf)-1] = '\0';
-			prt(buf,0,0);
+#ifdef JP
+			prt(format("%s ('r': פ)", buf),0,0);
+#else
+			prt(format("%s ('r' to recall)", buf),0,0);
+#endif
 
 			/* HACK : Add the line to message buffer */
 			message_add(buf);
@@ -3204,10 +2898,22 @@ bool probing(void)
 			lore_do_probe(i);
 
 			if (m_ptr->ml) move_cursor_relative(m_ptr->fy, m_ptr->fx);
-			inkey();
+			key = inkey();
 
 			Term_erase(0, 0, 255);
 
+			if (key == 'r')
+			{
+				bool oldcheat = cheat_know;
+
+				cheat_know = TRUE;
+				screen_roff(m_ptr->r_idx, 1);
+				cheat_know = oldcheat;
+
+				(void)inkey();
+				do_cmd_redraw();
+			}
+
 			/* Probe worked */
 			probe = TRUE;
 		}
@@ -3243,6 +2949,8 @@ bool destroy_area(int y1, int x1, int r, int full)
 	cave_type *c_ptr;
 	bool      flag = FALSE;
 
+	/* Unused */
+	(void)full;
 
 	/* Prevent destruction of quest levels and town */
 	if ((p_ptr->inside_quest && quest[p_ptr->inside_quest].type != QUEST_TYPE_RANDOM) || !dun_level)
@@ -3303,7 +3011,7 @@ bool destroy_area(int y1, int x1, int r, int full)
 			/* Preserve Artifacts */
 			if (ironman_hengband && preserve_mode)
 			{
-				s16b this_o_idx, next_o_idx = 0;
+				s16b this_o_idx, next_o_idx;
 
 				/* Scan all objects in the grid */
 				for (this_o_idx = c_ptr->o_idx; this_o_idx; this_o_idx = next_o_idx)
@@ -3386,6 +3094,7 @@ msg_print("ǳ褦ȯ");
 		}
 	}
 
+	forget_flow();
 
 	/* Mega-Hack -- Forget the view and lite */
 	p_ptr->update |= (PU_UN_VIEW | PU_UN_LITE);
@@ -3433,7 +3142,7 @@ bool earthquake_aux(int cy, int cx, int r, int m_idx)
 	bool            hurt = FALSE;
 	cave_type       *c_ptr;
 	bool            map[32][32];
-	char            *s;
+	cptr            s=NULL;
 
 
 	/* Prevent destruction of quest levels and town */
@@ -4483,7 +4192,7 @@ bool project_hook(int typ, int dir, int dam, u16b flg)
  */
 bool fire_bolt(int typ, int dir, int dam)
 {
-	u16b flg = PROJECT_STOP | PROJECT_KILL;
+	u16b flg = PROJECT_STOP | PROJECT_KILL | PROJECT_REFLECTABLE;
 	return (project_hook(typ, dir, dam, flg));
 }
 
@@ -4495,7 +4204,7 @@ bool fire_bolt(int typ, int dir, int dam)
  */
 bool fire_beam(int typ, int dir, int dam)
 {
-	u16b flg = PROJECT_BEAM | PROJECT_KILL;
+	u16b flg = PROJECT_BEAM | PROJECT_GRID | PROJECT_KILL;
 	return (project_hook(typ, dir, dam, flg));
 }
 
@@ -4528,7 +4237,7 @@ bool lite_line(int dir)
 
 bool drain_life(int dir, int dam)
 {
-	u16b flg = PROJECT_STOP | PROJECT_KILL;
+	u16b flg = PROJECT_STOP | PROJECT_KILL | PROJECT_REFLECTABLE;
 	return (project_hook(GF_OLD_DRAIN, dir, dam, flg));
 }
 
@@ -4563,77 +4272,77 @@ bool disarm_trap(int dir)
 
 bool heal_monster(int dir)
 {
-	u16b flg = PROJECT_STOP | PROJECT_KILL;
+	u16b flg = PROJECT_STOP | PROJECT_KILL | PROJECT_REFLECTABLE;
 	return (project_hook(GF_OLD_HEAL, dir, damroll(4, 6), flg));
 }
 
 
 bool speed_monster(int dir)
 {
-	u16b flg = PROJECT_STOP | PROJECT_KILL;
+	u16b flg = PROJECT_STOP | PROJECT_KILL | PROJECT_REFLECTABLE;
 	return (project_hook(GF_OLD_SPEED, dir, p_ptr->lev, flg));
 }
 
 
 bool slow_monster(int dir)
 {
-	u16b flg = PROJECT_STOP | PROJECT_KILL;
+	u16b flg = PROJECT_STOP | PROJECT_KILL | PROJECT_REFLECTABLE;
 	return (project_hook(GF_OLD_SLOW, dir, p_ptr->lev, flg));
 }
 
 
 bool sleep_monster(int dir)
 {
-	u16b flg = PROJECT_STOP | PROJECT_KILL;
+	u16b flg = PROJECT_STOP | PROJECT_KILL | PROJECT_REFLECTABLE;
 	return (project_hook(GF_OLD_SLEEP, dir, p_ptr->lev, flg));
 }
 
 
 bool stasis_monster(int dir)
 {
-	u16b flg = PROJECT_STOP | PROJECT_KILL;
+	u16b flg = PROJECT_STOP | PROJECT_KILL | PROJECT_REFLECTABLE;
 	return (project_hook(GF_STASIS, dir, p_ptr->lev, flg));
 }
 
 
 bool confuse_monster(int dir, int plev)
 {
-	u16b flg = PROJECT_STOP | PROJECT_KILL;
+	u16b flg = PROJECT_STOP | PROJECT_KILL | PROJECT_REFLECTABLE;
 	return (project_hook(GF_OLD_CONF, dir, plev, flg));
 }
 
 
 bool stun_monster(int dir, int plev)
 {
-	u16b flg = PROJECT_STOP | PROJECT_KILL;
+	u16b flg = PROJECT_STOP | PROJECT_KILL | PROJECT_REFLECTABLE;
 	return (project_hook(GF_STUN, dir, plev, flg));
 }
 
 
 bool poly_monster(int dir)
 {
-	u16b flg = PROJECT_STOP | PROJECT_KILL;
+	u16b flg = PROJECT_STOP | PROJECT_KILL | PROJECT_REFLECTABLE;
 	return (project_hook(GF_OLD_POLY, dir, p_ptr->lev, flg));
 }
 
 
 bool clone_monster(int dir)
 {
-	u16b flg = PROJECT_STOP | PROJECT_KILL;
+	u16b flg = PROJECT_STOP | PROJECT_KILL | PROJECT_REFLECTABLE;
 	return (project_hook(GF_OLD_CLONE, dir, 0, flg));
 }
 
 
 bool fear_monster(int dir, int plev)
 {
-	u16b flg = PROJECT_STOP | PROJECT_KILL;
+	u16b flg = PROJECT_STOP | PROJECT_KILL | PROJECT_REFLECTABLE;
 	return (project_hook(GF_TURN_ALL, dir, plev, flg));
 }
 
 
 bool death_ray(int dir, int plev)
 {
-	u16b flg = PROJECT_STOP | PROJECT_KILL;
+	u16b flg = PROJECT_STOP | PROJECT_KILL | PROJECT_REFLECTABLE;
 	return (project_hook(GF_DEATH_RAY, dir, plev * 200, flg));
 }
 
@@ -5242,7 +4951,11 @@ bool charge_monster(void)
 
 	if (!cave[y][x].m_idx)
 	{
+#ifdef JP
 		msg_print("ˤϥ󥹥Ϥޤ");
+#else
+		msg_print("You don't see any monster in this direction");
+#endif
 		return FALSE;
 	}
 
