@@ -743,6 +743,7 @@ static void mass_produce(object_type *o_ptr)
 		case TV_LIFE_BOOK:
 		case TV_SORCERY_BOOK:
 		case TV_MUSOU_BOOK:
+		case TV_MAGIC_BOOK:
 		{
 			if (cost <= 50L) size += damroll(2, 3);
 			if (cost <= 500L) size += damroll(1, 3);
@@ -1155,6 +1156,7 @@ static bool store_will_buy(object_type *o_ptr)
 			{
 				case TV_SORCERY_BOOK:
 				case TV_MUSOU_BOOK:
+				case TV_MAGIC_BOOK:
 				case TV_AMULET:
 				case TV_RING:
 				case TV_STAFF:
@@ -3632,6 +3634,8 @@ msg_format("%s $%ldѤޤ", o_name, (long)price);
 			display_inventory();
 		}
 	}
+
+	if (item == INVEN_WIELD) swap_wielding_hands();
 }
 
 
