@@ -1000,6 +1000,7 @@ struct player_type
 
 	s16b resist_magic;  /* Timed Resist Magic (later) */
 	s16b tim_radar;     /* Timed Radar Eye */
+	s16b tim_sentence;  /* Magic: Revenge Sentence */
 
 	s16b tim_xtra1;     /* Later */
 	s16b tim_xtra2;     /* Later */
@@ -1007,7 +1008,6 @@ struct player_type
 	s16b tim_xtra4;     /* Later */
 	s16b tim_xtra5;     /* Later */
 	s16b tim_xtra6;     /* Later */
-	s16b tim_xtra7;     /* Later */
 
 	s16b chaos_patron;  /* Old one */
 
@@ -1165,6 +1165,12 @@ struct player_type
 
 	s16b pspeed;		/* Current speed */
 
+	s16b keep_magic;	/* Magic: number of keeping spells */
+	u32b keep_spells;	/* Magic: keep spells */
+	s16b rvs_x;			/* Magic: Revenge Sentence - x */
+	s16b rvs_y;			/* Magic: Revenge Sentence - y */
+	s32b rvs_d;			/* Magic: Revenge Sentence - damage */
+
 	bool dtrap;
 	bool autopick_autoregister; /* auto register is in-use or not */
 
@@ -1399,3 +1405,16 @@ typedef struct {
 	byte dice;          /* Weapons which have more than 'dice' dice match */
 	byte bonus;         /* Items which have more than 'bonus' magical bonus match */
 } autopick_type;
+
+#ifdef TRAVEL
+/*
+ *  A structure type for travel command
+ */
+typedef struct {
+	int run;
+	int cost[MAX_HGT][MAX_WID];
+	int x;
+	int y;
+	int dir;
+} travel_type;
+#endif
