@@ -1165,3 +1165,8 @@ int server_port;
 
 /* for movie */
 bool browsing_movie;
+
+#ifdef TRAVEL
+/* for travel */
+travel_type travel;
+#endif
