@@ -124,6 +124,7 @@ static grouper group_item[] =
 	{ TV_LIFE_BOOK,     "ˡ̿" },
 	{ TV_SORCERY_BOOK,  "ˡѡ" },
 	{ TV_MUSOU_BOOK,    "ˡ̵С" },
+	{ TV_MAGIC_BOOK,    "ˡʼѡ" },
 
 	{ TV_CHEST,         "Ȣ" },
 	{ TV_FIGURINE,      "ͷ" },
@@ -169,6 +170,7 @@ static grouper group_item[] =
 	{ TV_LIFE_BOOK,     "Books (Life)" },
 	{ TV_SORCERY_BOOK,  "Books (Sorcery)" },
 	{ TV_MUSOU_BOOK,    "Books (Combat)" },
+	{ TV_MAGIC_BOOK,    "Books (Magic)" },
 
 	{ TV_CHEST,         "Chests" },
 	{ TV_FIGURINE,      "Magical Figurines" },
@@ -2777,7 +2779,7 @@ static void spoil_mon_info(cptr fname)
 			for (i = 0; i < vn; i++)
 			{
 #ifdef JP
-				if ( n != 0 ) spoil_out("");
+				if ( i != 0 ) spoil_out("");
 #else
 				if (!i) spoil_out(" resists ");
 				else if (i < vn-1) spoil_out(", ");
@@ -2841,7 +2843,7 @@ static void spoil_mon_info(cptr fname)
 			for (i = 0; i < vn; i++)
 			{
 #ifdef JP
-				if ( n != 0 ) spoil_out("");
+				if ( i != 0 ) spoil_out("");
 #else
 				if (!i) spoil_out(" resists ");
 				else if (i < vn-1) spoil_out(", ");
