@@ -65,6 +65,504 @@ void do_cmd_rerate(bool flag)
 }
 
 
+static cptr ltrim(cptr p)
+{
+	while(p[0] == ' ') p++;
+	return p;
+}
+
+static void rtrim(char *p)
+{
+	int i = strlen(p) - 1;
+	while(p[i] == ' ') p[i--] = '\0';
+}
+
+/* Reversive strncmp */
+static int strrncmp(cptr s1, cptr s2, int len)
+{
+	int i;
+	int l1 = strlen(s1);
+	int l2 = strlen(s2);
+
+	for (i = 1; i <= len; i++)
+	{
+		int p1 = l1 - i;
+		int p2 = l2 - i;
+
+		if (l1 != l2)
+		{
+			if (p1 < 0) return (-1);
+			if (p2 < 0) return (1);
+		}
+		else
+		{
+			if (p1 < 0) return (0);
+		}
+
+		if (s1[p1] < s2[p2]) return (-1);
+		if (s1[p1] > s2[p2]) return (-1);
+	}
+
+	return (0);
+}
+
+/*
+ * XAngband: wishing
+ * Make an wishing object, ego or artifact when it exists.
+ * Return values:
+ * 0 - canceled
+ * 1 - normal objects
+ * 2 - ego items
+ * 3 - artifacts
+ * -1 - failed
+ */
+s16b do_cmd_wishing(int prob, bool art, bool ego)
+{
+	int k, i;
+	int a_id, a_num;
+	int e_id[10], e_num;
+	int k_id, k_num;
+	char buf[MAX_NLEN];
+	char o_name[MAX_NLEN];
+	cptr str;
+	object_type *q_ptr;
+	object_kind *k_ptr;
+	artifact_type *a_ptr;
+	ego_item_type *e_ptr;
+
+	/* initialize */
+	bool wish_art = FALSE;
+	bool randart = FALSE;
+	bool wish_ego = FALSE;
+	bool base = TRUE;
+	bool ok = (randint0(100) < prob) ? 1 : 0;
+
+	buf[0] = '\0';
+	str = buf;
+
+	/* get wishing */
+#ifdef JP
+	if (!get_string("˾ߡ ", buf, (MAX_NLEN - 1))) return (0);
+#else
+	if (!get_string("For what do you wish? ", buf, (MAX_NLEN - 1))) return (0);
+#endif
+
+#ifndef JP
+	str_tolower(buf);
+
+	/* remove 'a' */
+	if (!strncmp(buf, "a ", 2)) str = ltrim(str + 1);
+	else if (!strncmp(buf, "an ", 3)) str = ltrim(str + 2);
+#endif
+
+	/* remove surplus spaces */
+	rtrim(buf);
+
+	/* evaluate header strings */
+#ifdef JP
+	/* wishing preserve artifacts ? */
+	if (!strncmp(str, "", 2))
+	{
+		str = ltrim(str + 2);
+		wish_art = TRUE;
+		base = FALSE;
+	}
+
+	/* wishing random artifacts ? */
+	else if (!strncmp(str, "", 2))
+	{
+		str = ltrim(str + 2);
+		wish_art = TRUE;
+		randart = TRUE;
+	}
+
+	/* wishing random ego ? */
+	else if (!strncmp(str, "", 6))
+	{
+		str = ltrim(str + 6);
+		wish_ego = TRUE;
+	}
+#else
+	/* wishing artifacts ? */
+	if (!strncmp(str, "the ", 4))
+	{
+		str = ltrim(str + 4);
+		wish_art = TRUE;
+		randart = TRUE;
+	}
+
+	/* wishing random ego ? */
+	else if (!strncmp(str, "excellent ", 9))
+	{
+		str = ltrim(str + 9);
+		wish_ego = TRUE;
+	}
+#endif
+
+	if (strlen(str) < 1)
+	{
+#ifdef JP
+		msg_print("̾ʤ");
+#else
+		msg_print("What?");
+#endif
+		return (0);
+	}
+
+	if ((!art) && (wish_art))
+	{
+#ifdef JP
+		msg_print("ƥեȤϴꤨʤ");
+#else
+		msg_print("You can not wish artifacts!");
+#endif
+		return (0);
+	}
+
+	if (cheat_xtra) msg_format("Wishing %s....", buf);
+
+	/* search normal items */
+	k_id = -1;
+	k_num = 0;
+
+	if (base)
+	{
+		int len;
+		int mlen = 0;
+
+		for (k = 1; k < max_k_idx; k++)
+		{
+			object_type forge;
+
+			q_ptr = &forge;
+			k_ptr = &k_info[k];
+
+			/* Skip "empty" objects */
+			if (!k_ptr->name) continue;
+
+			/* make object */
+			object_prep(q_ptr, k);
+
+			/* get name */
+			object_desc(o_name, q_ptr, (OD_OMIT_PREFIX | OD_NAME_ONLY | OD_STORE));
+#ifndef JP
+			str_tolower(o_name);
+#endif
+
+			if (cheat_xtra) msg_format("Matching object No.%d %s", k, o_name);
+
+			/* matching */
+			len = strlen(o_name);
+#ifdef JP
+			if (!strrncmp(str, o_name, len))
+#else
+			if (!strncmp(str, o_name, len))
+#endif
+			{
+				/* Choice a object which matches longer */
+				if (len > mlen)
+				{
+					k_id = k;
+					k_num = 1;
+					mlen = len;
+				}
+			}
+		}
+
+		/* if wishing object exists, test ego name */
+		if ((ego) && (k_num == 1))
+		{
+			e_num = 0;
+
+			for (k = 1; k < max_e_idx; k++)
+			{
+				e_ptr = &e_info[k];
+
+				/* Skip "empty" ego */
+				if (!e_ptr->name) continue;
+
+				/* get name */
+				strcpy(o_name, (e_name + e_ptr->name));
+#ifndef JP
+				str_tolower(o_name);
+#endif
+
+				if (cheat_xtra) msg_format("Mathcing ego No.%d %s...", k, o_name);
+
+				/* matching */
+#ifdef JP
+				if (!strncmp(str, o_name, strlen(o_name)))
+#else
+				if (!strrncmp(str, o_name, strlen(o_name)))
+#endif
+				{
+					/* memorize same named egos */
+					e_id[e_num++] = k;
+				}
+			}
+		}
+	}
+
+	/* search artifacts */
+	a_id = -1;
+	a_num = 0;
+
+	if (art)
+	{
+		char *astr;
+		char aname[MAX_NLEN];
+
+		for (k = 1; k < max_a_idx; k++)
+		{
+			object_type forge;
+
+			q_ptr = &forge;
+			a_ptr = &a_info[k];
+
+			/* Skip "empty" artifacts */
+			if (!a_ptr->name) continue;
+
+			/* make fake artfact */
+			i = lookup_kind(a_ptr->tval, a_ptr->sval);
+			if (!i) continue;
+			object_prep(q_ptr, i);
+			q_ptr->name1 = k;
+
+			/* get name */
+			object_desc(o_name, q_ptr, (OD_OMIT_PREFIX | OD_NAME_ONLY | OD_STORE));
+#ifndef JP
+			str_tolower(o_name);
+#endif
+
+			/* get omitted name */
+			astr = (a_name + a_ptr->name);
+
+			/* remove full-name flag */
+			if (astr[0] == '$') astr++;
+
+#ifdef JP
+			/* remove quotes */
+			if (!strncmp(astr, "", 2))
+			{
+				strcpy(aname, (astr + 2));
+				astr = strstr(aname, "");
+				astr[0] = '\0';
+			}
+			/* remove 'of' */
+			else
+			{
+				int l = strlen(astr);
+				strcpy(aname, astr);
+				if (!strrncmp(aname, "", 2))
+				{
+					aname[l-2] = '\0';
+				}
+			}
+#else
+			/* remove quotes */
+			if (astr[0] == '\'')
+			{
+				strcpy(aname, (astr + 1));
+				astr = strchr(aname, '\'');
+				astr[0] = '\0';
+			}
+			/* remove 'of ' */
+			else if (!strncmp(astr, "of ", 3))
+			{
+				strcpy(aname, (astr + 3));
+			}
+
+			str_tolower(aname);
+#endif
+
+#ifdef JP
+			if (cheat_xtra) msg_format("Matching artifact No.%d %s(%s)", k, aname, &o_name[2]);
+#else
+			if (cheat_xtra) msg_format("Matching artifact No.%d %s(%s)", k, aname, o_name);
+#endif
+
+			/* entire matched */
+#ifdef JP
+			if (!strcmp(&o_name[2], str))
+#else
+			if (!strcmp(o_name, str))
+#endif
+			{
+				a_id = k;
+				a_num = 1;
+				break;
+			}
+
+			/* partial matched */
+			else if (!strcmp(aname, str))
+			{
+				if (one_in_(a_num))
+				{
+					a_id = k;
+					a_num++;
+				}
+			}
+		}
+	}
+
+	/* Too many matches */
+	if ((wizard) && ((a_num > 1) || (k_num > 1)))
+	{
+#ifdef JP
+		msg_print("䤬¿롪");
+#else
+		msg_print("Too many matches!");
+#endif
+		return (-1);
+	}
+
+	/* wished artifact is found */
+	else if ((a_id >= 0) && (a_num == 1))
+	{
+		if ((prob < 0) || (ok && !a_info[a_id].cur_num))
+		{
+			/* make target preserve artifact */
+			create_named_art(a_id, py, px);
+		}
+		else
+		{
+#ifdef JP
+			msg_print("­žäƤΤ褦˾äƤޤä");
+#else
+			msg_print("You feel something roll beneath your feet, but it disappears in a puff of smoke!");
+#endif
+		}
+
+		return (3);
+	}
+
+	else if ((!ego) && (wish_ego || e_num))
+	{
+#ifdef JP
+		msg_print("ƥϴꤨʤ");
+#else
+		msg_print("Can not wish ego items.");
+#endif
+		return (0);
+	}
+
+	/* wished object is found */
+	else if ((k_id >= 0) && (k_num == 1))
+	{
+		byte retval = 1;
+		int object_level;
+		object_type forge;
+		q_ptr = &forge;
+		k_ptr = &k_info[k_id];
+		object_level = k_ptr->level;
+
+		/* make object */
+		object_prep(q_ptr, k_id);
+
+		if ((q_ptr->tval == TV_STAFF) && (q_ptr->sval == SV_STAFF_WISHING))
+		{
+#ifdef JP
+			msg_print("δꤤϤǤˤʤäƤ롣");
+#else
+			msg_print("This wish has already been fulfilled.");
+#endif
+			return (-1);
+		}
+		else if (k_ptr->gen_flags & (TRG_INSTA_ART))
+		{
+			for (k = 0; k < max_a_idx; k++)
+			{
+				a_ptr = &a_info[k];
+				if (a_ptr->tval != k_ptr->tval) continue;
+				if (a_ptr->sval != k_ptr->sval) continue;
+				q_ptr->name1 = k;
+				break;
+			}
+
+			if (!a_ptr->cur_num)
+			{
+				/* Apply magic */
+				apply_magic(q_ptr, -1, TRUE, TRUE, TRUE, FALSE);
+			}
+			else
+			{
+#ifdef JP
+				msg_print("­žäƤΤ褦˾äƤޤä");
+	#else
+				msg_print("You feel something roll beneath your feet, but it disappears in a puff of smoke!");
+#endif
+			}
+
+			retval = 3;
+		}
+		else if (randart)
+		{
+			(void)create_artifact(q_ptr, FALSE);
+			retval = 3;
+		}
+		else if ((ego) && (wish_ego || e_num))
+		{
+			int max_roll = 1000;
+
+			for (i = 0; i < max_roll; i++)
+			{
+				(void)apply_magic(q_ptr, object_level, FALSE, TRUE, TRUE, FALSE);
+
+				if (q_ptr->name1)	/* Paranoia */
+				{
+					object_prep(q_ptr, k_id);
+					continue;
+				}
+
+				/* wishing a random ego */
+				if (wish_ego) break;
+
+				for (k = 0; k < e_num; k++)
+				{
+					if (q_ptr->name2 == e_id[k]) break;
+				}
+				if (k < e_num) break;
+
+				object_prep(q_ptr, k_id);
+			}
+
+			if (i == max_roll)
+			{
+#ifdef JP
+				msg_print("ԡ⤦ٴäƤߤƤ");
+#else
+				msg_print("Failed! Try again.");
+#endif
+				return (-1);
+			}
+
+			retval = 2;
+		}
+		else
+		{
+			apply_magic(q_ptr, -1, FALSE, FALSE, FALSE, FALSE);
+		}
+
+		(void)drop_near(q_ptr, -1, py, px);
+
+		return (retval);
+	}
+
+	/* Nothing */
+	else
+	{
+#ifdef JP
+		msg_print("󡢤ʤΤ¸ߤʤ褦");
+#else
+		msg_print("Ummmm, that is not existing.");
+#endif
+		return (-1);
+	}
+
+	return (1);
+}
+
+
 #ifdef ALLOW_WIZARD
 
 /*
@@ -626,6 +1124,7 @@ static tval_desc tvals[] =
 	{ TV_LIFE_BOOK,         "ˡ(̿)"         },
 	{ TV_SORCERY_BOOK,      "ˡ()"         },
 	{ TV_MUSOU_BOOK,        "ˡ(̵)"         },
+	{ TV_MAGIC_BOOK,        "ˡ()"         },
 	{ TV_SPIKE,             ""               },
 	{ TV_DIGGING,           "ηƻ"             },
 	{ TV_CHEST,             "Ȣ"                 },
@@ -665,6 +1164,7 @@ static tval_desc tvals[] =
 	{ TV_LIFE_BOOK,         "Life Spellbook"       },
 	{ TV_SORCERY_BOOK,      "Sorcery Spellbook"    },
 	{ TV_MUSOU_BOOK,        "Combat Spellbook"     },
+	{ TV_MAGIC_BOOK,        "Magic Spellbook"     },
 	{ TV_SPIKE,             "Spikes"               },
 	{ TV_DIGGING,           "Digger"               },
 	{ TV_CHEST,             "Chest"                },
@@ -1992,6 +2492,11 @@ void do_cmd_debug(void)
 		do_cmd_wiz_hack_ben();
 		break;
 
+	/* Hack: wishing */
+	case 'W':
+		do_cmd_wishing(-1, TRUE, TRUE);
+		break;
+
 	/* Not a Wizard Command */
 	default:
 		msg_print("That is not a valid debug command.");
@@ -2007,4 +2512,3 @@ static int i = 0;
 #endif
 
 #endif
-
