@@ -245,6 +245,18 @@ static void prt_stat(int stat)
 #define BAR_INFRAVISION 26
 #define BAR_RECALL 27
 #define BAR_RADAREYE 28
+#define BAR_SHCOLD 29
+#define BAR_EXMIGHT 30
+#define BAR_BUILDUP 31
+#define BAR_ANTITELE 32
+#define BAR_ANTIMAGIC 33
+#define BAR_EYEFOREYE 34
+#define BAR_RESDARK 35
+#define BAR_RESNETH 36
+#define BAR_SENTENCE 37
+#define BAR_RUNESWORD 38
+#define BAR_ANTIMULTI 39
+#define BAR_VAMPIRIC 40
 
 
 static struct {
@@ -283,6 +295,18 @@ static struct {
 	{TERM_L_RED, "", "ֳ"},
 	{TERM_WHITE, "", ""},
 	{TERM_L_GREEN, "", "졼"},
+	{TERM_WHITE, "", "ɹ"},
+	{TERM_YELLOW, "", "϶"},
+	{TERM_RED, "", "ζ"},
+	{TERM_ORANGE, "", "ȿƥ"},
+	{TERM_RED, "", "ȿˡ"},
+	{TERM_SLATE, "", "ܤˤܤ"},
+	{TERM_L_DARK, "", "ѰŹ"},
+	{TERM_L_DARK, "", "Ϲ"},
+	{TERM_SLATE, "", ""},
+	{TERM_L_DARK, "", ""},
+	{TERM_L_DARK, "", "ȿ"},
+	{TERM_RED, "", "۷Ƿ"},
 	{0, NULL, NULL}
 };
 #else
@@ -316,6 +340,18 @@ static struct {
 	{TERM_L_RED, "If", "Infr"},
 	{TERM_WHITE, "Rc", "Recall"},
 	{TERM_L_GREEN, "Rd", "Rader"},
+	{TERM_WHITE, "SCo", "SCold"},
+	{TERM_YELLOW, "EMi", "ExMight"},
+	{TERM_RED, "Bu", "BuildUp"},
+	{TERM_ORANGE, "AT", "AntiTele"},
+	{TERM_RED, "AM", "AntiMagic"},
+	{TERM_SLATE, "Ey", "Eye-Eye"},
+	{TERM_L_DARK, "Dk", "Dark"},
+	{TERM_L_DARK, "Nt", "Nether"},
+	{TERM_SLATE, "Rv", "Revenge"},
+	{TERM_L_DARK, "Rs", "RuneSword"},
+	{TERM_L_DARK, "AMl", "AntiMulti"},
+	{TERM_RED, "Vm", "Vampiric"},
 	{0, NULL, NULL}
 };
 #endif
@@ -329,7 +365,7 @@ static struct {
  */
 static void prt_status(void)
 {
-	u32b bar_flags[1];
+	u32b bar_flags[2];
 	int wid, hgt, row_statbar, max_col_statbar;
 	int i, col = 0, num = 0;
 	int space = 2;
@@ -341,6 +377,7 @@ static void prt_status(void)
 	Term_erase(0, row_statbar, max_col_statbar);
 
 	bar_flags[0] = 0L;
+	bar_flags[1] = 0L;
 
 	/* Hallucinating */
 	if (p_ptr->image) ADD_FLG(BAR_HALLUCINATION);
@@ -388,9 +425,11 @@ static void prt_status(void)
 
 	/* Super Heroism / berserk */
 	if (p_ptr->shero) ADD_FLG(BAR_BERSERK);
+	if (is_keeping_spell(MS_BERSERK)) ADD_FLG(BAR_BERSERK);
 
 	/* Blessed */
 	if (p_ptr->blessed) ADD_FLG(BAR_BLESSED);
+	if (is_keeping_spell(MS_BLESSING)) ADD_FLG(BAR_BLESSED);
 
 	/* Shield */
 	if (p_ptr->magicdef) ADD_FLG(BAR_MAGICDEFENSE);
@@ -426,8 +465,48 @@ static void prt_status(void)
 	/* Ultimate-resistance */
 	if (p_ptr->musou) ADD_FLG(BAR_ULTIMATE);
 
-	/* tim stealth */
+	/* Fire Aura */
 	if (p_ptr->tim_sh_fire) ADD_FLG(BAR_SHFIRE);
+	if (is_keeping_spell(MS_DEMON_AURA)) ADD_FLG(BAR_SHFIRE);
+
+	/* Magic: Cold Aura */
+	if (is_keeping_spell(MS_ICE_ARMOR)) ADD_FLG(BAR_SHCOLD);
+
+	/* Magic: Extra Might */
+	if (is_keeping_spell(MS_EXTRA_MIGHT)) ADD_FLG(BAR_EXMIGHT);
+
+	/* Magic: Build Up */
+	if (is_keeping_spell(MS_BUILD_UP)) ADD_FLG(BAR_BUILDUP);
+
+	/* Magic: Anti Teleport Field */
+	if (is_keeping_spell(MS_ANTI_TELEPORT)) ADD_FLG(BAR_ANTITELE);
+
+	/* Magic: Anti Magic Field */
+	if (is_keeping_spell(MS_ANTI_MAGIC)) ADD_FLG(BAR_ANTIMAGIC);
+
+	/* Magic: Eye for an Eye */
+	if (is_keeping_spell(MS_EYE_FOR_EYE)) ADD_FLG(BAR_EYEFOREYE);
+
+	/* Magic: Resist Dark */
+	if (is_keeping_spell(MS_RES_DARK)) ADD_FLG(BAR_RESDARK);
+
+	/* Magic: Resist Dark */
+	if (is_keeping_spell(MS_RES_NETHER)) ADD_FLG(BAR_RESNETH);
+
+	/* Magic: Revenge Sentence */
+	if (p_ptr->tim_sentence) ADD_FLG(BAR_SENTENCE);
+
+	/* Magic: Rune sword */
+	if (is_keeping_spell(MS_RUNESWORD)) ADD_FLG(BAR_RUNESWORD);
+
+	/* Magic; Vampiric Attack */
+	if (is_keeping_spell(MS_VAMPIRIC_ATTACK)) ADD_FLG(BAR_VAMPIRIC);
+
+	/* Magic: Confusing Hands */
+	if (is_keeping_spell(MS_CONFUSION)) ADD_FLG(BAR_ATTKCONF);
+
+	/* Magic; Anti Multiply */
+	if (is_keeping_spell(MS_ANTI_MULTIPLY)) ADD_FLG(BAR_ANTIMULTI);
 
 	/* Calcurate length */
 	for (i = 0; bar[i].sstr; i++)
@@ -994,6 +1073,19 @@ static void prt_state(void)
 		}
 	}
 
+	/* Magic: Keep casting */
+	else if (p_ptr->keep_magic)
+	{
+#ifdef JP
+		strcpy(text, "Ӿ");
+#else
+		strcpy(text, "Spell");
+#endif
+		if (p_ptr->keep_magic == 2) attr = TERM_YELLOW;
+		if (p_ptr->keep_magic == 3) attr = TERM_ORANGE;
+		if (p_ptr->keep_magic >= 4) attr = TERM_L_RED;
+	}
+
 	/* Searching */
 	else if (p_ptr->searching)
 	{
@@ -2271,6 +2363,9 @@ static void calc_hitpoints(void)
 	if (p_ptr->hero) mhp += 10;
 	if (p_ptr->shero) mhp += 30;
 
+	/* Magic: in build-up */
+	if (is_keeping_spell(MS_BUILD_UP)) mhp += 100;
+
 	/* New maximum hitpoints */
 	if (p_ptr->mhp != mhp)
 	{
@@ -2451,6 +2546,7 @@ static sint add_special_melee_skill(byte pclass, s16b weight, object_type *o_ptr
 
 		/* Priest.  Can use 12 lb weapons without penalty at level 1, and 22 lb weapons without penalty at 50th level. */
 		case CLASS_PRIEST:
+		case CLASS_DEVICE_USER:
 		{
 			add_skill = 30 + (1 * p_ptr->lev / 2) - (weight / 4);
 			if (add_skill > 0) add_skill = 0;
@@ -3114,6 +3210,10 @@ void calc_bonuses(void)
 	}
 
 
+	/* Magic */
+	calc_bonuses_magic_spell();
+
+	
 	/* Scan the usable inventory */
 	for (i = INVEN_WIELD; i < INVEN_TOTAL; i++)
 	{
@@ -3427,7 +3527,7 @@ void calc_bonuses(void)
 	}
 
 	/* Temporary blessing */
-	if (p_ptr->blessed)
+	if ((p_ptr->blessed) || (is_keeping_spell(MS_BLESSING)))
 	{
 		p_ptr->to_a += 5;
 		p_ptr->dis_to_a += 5;
@@ -3500,7 +3600,7 @@ void calc_bonuses(void)
 	}
 
 	/* Temporary "Beserk" */
-	if (p_ptr->shero)
+	if ((p_ptr->shero) || (is_keeping_spell(MS_BERSERK)))
 	{
 		p_ptr->to_h += 24;
 		p_ptr->dis_to_h += 24;
@@ -3769,6 +3869,9 @@ void calc_bonuses(void)
 					num = cp_ptr->num; wgt = cp_ptr->wgt; mul = cp_ptr->mul;
 			}
 
+			/* Magic: Build up increases 1 more blows */
+			if (is_keeping_spell(MS_BUILD_UP)) num++;
+
 			/* Enforce a minimum "weight" (tenth pounds) */
 			div = ((o_ptr->weight < wgt) ? wgt : o_ptr->weight);
 
